/* **********************************************************
   File Milestone.h
   It defines and implements the basic unit of the PRM, a 
   Milestone. In this case it is composed by the configuration of
   the robot, the link to its father and their childs, as well as
   a unique identifier for each of them

   Gildardo Sanchez-Ante
   Robotics Lab., Stanford University
*************************************************************/
#ifndef MILESTONE_H
#define MILESTONE_H

#include <iostream.h>
#include <stdio.h>
#include <vector.h>

#include "GenCoords.h"
#include "Edge.h"
class Edge;

class Milestone {

 public:

  // Constructors
  Milestone(){ 
    i = 0; t =0;
    edgeFromFather.setEdge(0,0,0,0);
  }

  // Function setRoot on the planner is the one that uses this
  Milestone(int inx, const GenCoords& qq, double tim=0) : q(qq){
    i = inx; t=tim;
    edgeFromFather.setEdge(-999, inx);
  } 

  void setIndex(const int& index){i=index;}
  void setConfig(const GenCoords& q_n){q=q_n;}
  void setTime(const double& time){t=time;}

  // This function is used to set that the current milestone is the
  // father of the milestone child.
  void isFatherOf(Milestone& child){
   child.edgeFromFather.setEdge(getIndex(),child.getIndex(),distance(child),0);
   addChild(child);
  }

  void setFather(int from, int to){
    edgeFromFather.setEdge(from, to, 0 ,0);
  }

  void addChild(const Milestone& m){
    children.push_back(m.getIndex());
  }

  
  GenCoords getConfig() const { return q; }
  int getIndex()const {return i;}
  double getTime() const { return t;}
  int getFather() {return edgeFromFather.getFrom();} 
  Edge& getEdge(){return edgeFromFather;}
  int getNrSons()const{return children.size();}
  int getChild(int k){return children[k];}
  void setKappa(int k){edgeFromFather.setKappa(k);}
  void markSafe(){edgeFromFather.markSafe();}

  void removeChild(const Milestone& m){   
    vector<int>::iterator I;
    for(I= children.begin();I!=children.end();++I)
      if((*I)== m.getIndex()){
	children.erase(I);
	break;
      }
  }

  void print()     { 
    printf("%i   %.4f  %.4f  %.4f  %.4f  %.4f  %.4f\n",
	   i, q[0],q[1],q[2],q[3],q[4],q[5]);
    cerr<<"This Node has "<<edgeFromFather.getFrom()<<" as father"<<endl;
    edgeFromFather.print();cerr<<"\n\n";
  }

  void printChilds(){ 
    vector<int>::iterator I;
    cerr<<"Sons of: "<<getIndex()<<"  ";
    for(I=children.begin();I!=children.end();++I)
      cerr<<(*I)<<", ";
    cerr<<endl;
  }
  
  //Euclidean Distance
  double distance(const Milestone& m) const{
    double sum = 0.0;
    for(int i=0;i<6;++i)
      sum+=((m.q[i]-q[i])*(m.q[i]-q[i]));
    return(sqrt(sum));
  }
  
  // Linf distance
  double L_inf(const Milestone& m) const{
    double max = 0.0;
    for(int i=0;i<6;++i)
      if(fabs(m.q[i]-q[i]) > max)
	max = fabs(m.q[i]-q[i]);
    return(max);
  }
  
  protected:
  
  int       i;     // an index to identify the nodes
  GenCoords q;     // Configuration
  double    t;     // time associated to the configuration
  Edge      edgeFromFather; // link to the father of the node
  vector<int> children; // to store who are the children of the node
};

istream& operator>> (istream& is, Milestone& m);
ostream& operator<< (ostream& os, const Milestone& m);

#endif
  








