/* Copyright (c) 1996 by Stanford University.  All Rights Reserved */

/***************************************************************************
   NAME
     RandDouble

   PURPOSE
     generate uniformly-distributed random numbers between [0, 1)

   NOTES
     The original code is due to D. Knuth. 

   AUTHOR
     David Hsu  
   
     dyhsu@cs.stanford.edu
     http://robotics.stanford.edu/~dyhsu 
                  
   HISTORY
     created 01/07/97

***************************************************************************/


#ifndef RANDDOUBLE_H
#define RANDDOUBLE_H

#include <sys/types.h>
#include <time.h>

#define CACHE_SIZE	1024		/* must be > KK_DOUBLE */
#define	KK_RD		100 


class RandDouble {
public:
  RandDouble();
  RandDouble(long seed);
  const double* get(int num);

protected:
  double cache[CACHE_SIZE];		/* cache of random numbers */ 
  double *cacheEnd;
  double *current;			
  double ran_double_x[KK_RD];     	/* the generator state     */

  void ranDoubleStart(long seed);
  void ranDoubleArray(double *aa, int n);
};


inline RandDouble::RandDouble() 
{
  current = cacheEnd = cache + CACHE_SIZE;
  long key = time(NULL);
  ranDoubleStart( key );
}

inline RandDouble::RandDouble(long seed)
{  
  current = cacheEnd = cache + CACHE_SIZE;
  ranDoubleStart(seed);
}

/* get
 *   return a pointer to an array of  random doubles
 *
 *   num: number of random doubles requested
 */

inline const double* RandDouble::get(int num)
{
  double* rd = current;
  if ( current+num >= cacheEnd ) {
    ranDoubleArray(cache, CACHE_SIZE);
    rd = current = cache;
  }
  current += num;
  return rd;
}

#undef CACHE_SIZE

#endif
