/* Copyright (c) 1996 by Stanford University.  All Rights Reserved */

/***************************************************************************
   NAME
     RandInt

   PURPOSE
     generate uniformly-distributed random numbers between [0, 2^30-1]

   NOTES
     The original code is due to D. Knuth. 

   AUTHOR
     David Hsu  
   
     dyhsu@cs.stanford.edu
     http://robotics.stanford.edu/~dyhsu 
                  
   HISTORY
     created 01/07/97

***************************************************************************/


#ifndef RANDINT_H
#define RANDINT_H

#include <sys/types.h>
#include <time.h>

#define CACHE_SIZE	1024		/* must be > 100 */
#define	KK_RI 		100    


class RandInt {
public:
  RandInt();
  RandInt(long seed);
  inline const  long* get(int num);

private:
  long cache[CACHE_SIZE];		/* cache of random numbers */ 
  long *cacheEnd;
  long *current;			
  long ran_x[KK_RI];	           	/* the generator state     */

  void ranStart(long seed);
  void ranArray(long* aa, int n);
};


inline RandInt::RandInt()
{
  current = cacheEnd = cache + CACHE_SIZE;
  long key = time(NULL);
  ranStart( key );
}

inline RandInt::RandInt(long seed)
{  
  current = cacheEnd = cache + CACHE_SIZE;
  ranStart(seed);
}

/* get
 *   return a pointer to an array of  random intergers
 *
 *   num: number of random intergers requested
 */

inline const long* RandInt::get(int num)
{
  long* rl = current;
  if ( current+num >= cacheEnd ) {
    ranArray(cache, CACHE_SIZE);
    rl = current = cache;
  }
  current += num;
  return rl;
}

#undef CACHE_SIZE

#endif
