#ifndef ROBOT_SIMULATOR_H
#define ROBOT_SIMULATOR_H

class GenCoords;
class Milestone;
class Pose;
class Path;


/* IntervalTyple
 *    
 *    different types of intervals
 */

enum IntervalType {Open=0, SemiOpen, SemiClosed, Closed};

/* RobotSimulator
 *
 *    base class for a robot
 *    encapsulate the internal structures of a robot and provide an 
 *    interface for planners
 */

class  RobotSimulator {

public:
  virtual ~RobotSimulator();

  //
  // collision detection
  //
  virtual bool   IsClear(const Milestone& m);
  virtual bool   IsClear(const Milestone& m0, const Milestone& m1, 
			 IntervalType i=Closed);
  virtual bool   IsClear(const Path& p);


  // 
  // distance in the workspace
  //
  virtual double WspDist(const GenCoords& p, const GenCoords& q);

  //
  // cost function
  //
  virtual double Cost(const GenCoords& q0, const GenCoords& q1);
  virtual double Cost(const Path& p);

  //
  // kinematics
  //
  virtual bool IK(const Pose& x, GenCoords *q, const GenCoords *q0=0);
  virtual void FK(const GenCoords& q, Pose *x, int r=0);

  //
  // path interpolation
  //
  virtual void InterpolatePath(const Path& p, Path *ip, double res);

  //Utilities
  virtual const Pose& GetBase();
  virtual void SetBase(const Pose& basePose_);

};


#endif 







