#ifndef joint_h
#define joint_h

#include <Inventor/fields/SoMFFloat.h>
#include <Inventor/nodes/SoTransform.h>

class GenCoords;
class JointEngine;
class JointInvEngine;

class Joint : public SoTransform {
  
  SO_NODE_ABSTRACT_HEADER(Joint);

public:

  // Fields:
  SoMFFloat val;
  SoMFFloat min;
  SoMFFloat max;
  SoMFFloat def;
  SoMFFloat vmax;

  // Methods:
  virtual bool getVals(GenCoords *dest, int &idx);
  virtual int  getActVals(double *dest, int &idx);
  virtual bool setVals(const GenCoords *src, int &idx);
  virtual int  setActVals(const double *src, int &idx);
  virtual int  getNumVals() {return val.getNum();}
  virtual double getValAt(int i);
  virtual double getActValAt(int i);
  virtual bool setValAt(int i, double newVal);
  virtual bool setActValAt(int i, double newVal);

  virtual bool getMins(GenCoords *dest, int &idx);
  virtual bool getMaxs(GenCoords *dest, int &idx);
  virtual double getMinAt(int i);
  virtual double getMaxAt(int i);
  
  virtual bool getVmaxs(GenCoords *dest, int &idx);
  virtual int  getActVmaxs(double *dest, int &idx);
  virtual double getVmaxAt(int i);
  virtual double getActVmaxAt(int i);

  virtual void connectInvEngine(SoTransform *xform) {}
  virtual void disconnectInvEngine() {}

  static void initClass();


protected:

  Joint();  
  virtual ~Joint();

	JointEngine *engine;
	JointInvEngine *invEngine;
};

#endif
  
