#ifndef link_h
#define link_h

#include <Inventor/fields/SoMFFloat.h>
#include <Inventor/nodes/SoSeparator.h>

#include <PQP.h>
#include "Obstacle.h"

class SoPath;
class SoPrimitiveVertex;
class JointKit;

class Link : public SoSeparator {
  
  SO_NODE_HEADER(Link);

public:

  // Fields:

  // Methods:
  static void initClass();

  void buildColModel(JointKit *parent_);
  PQP_Model *getColModel();

  // Remember that PQP Model does not contain the info on the position
  // of the object, so , it should be sent independently
  void setColTransform();
  void setColTransform(PQP_REAL rot[3][3], PQP_REAL trans[3]);

  double getCurDist(Obstacle *obs, double tol_ = 0);
  bool isClear(Obstacle *obs);
  bool isClear(Link* link2);

  static void _TriangulateCB(void *data, SoCallbackAction *,
			     const SoPrimitiveVertex *vertex1,
			     const SoPrimitiveVertex *vertex2,
			     const SoPrimitiveVertex *vertex3);
    
 public:
  
  Link();  
  virtual ~Link();
  
  // Each link is going to be represented by its PQP Model
  PQP_Model *colModel;
  PQP_Model *oldObjModel;

  // And its rotation matrix and translation vector
  PQP_REAL rot[3][3];
  PQP_REAL trans[3];

  JointKit *parent;
  
  double dist;
  double tol;
};

#endif
