// *******************************************
// File MY_TREE.h
// Defines the class of Single-Query Probabilistic 
// RoadMaps

// This class is the responsible of maintain the 
// record of all nodes added so far as well as to
// keep a representation of the sampled C-space 
// in order to choose a node to expand according 
// to the density of nodes.
// ********************************************

#ifndef MY_TREE_H
#define MY_TREE_H

#include <vector.h>
#include <queue>

#include "GenCoords.h"
#include "Milestone.h"
#include "Path.h"
#include "Bin.h"
#include "Edge.h"

class myTree {
 public:

  myTree();
  ~myTree();

  void setGridSize(int a=10, int b=10);
  void setDOFS(int dof0=0, int dof1=1)
    {DOF0=dof0; DOF1=dof1;};
  int getNodeToExpand();
  void addToBins(const Milestone& m);
  void delFromBins(const Milestone& m);
  int chooseBin();
  bool hasNodesIn(int bin);
  int getNodeFromBin(int bin);
  double getRatio();
  int getClosestNode(const Milestone& m, Milestone x[12000]);
  int getBinIndex(const Milestone& m);
  int getSize(){return nodes_on_tree;}
  void printBins();

  double getDelta(){return DELTA;}
  int getDIVS(){return DIVS;}

  void rebuildTree(myTree& t, Milestone theNodes[12000]);

 protected:

  Bin binList[1000];    // Array for the bins 
  int filled_bins[1000];   // For the valid bins

  static bool initRand;      // to initialize the random seed

  int nodes_on_tree;           
  int bins_ctr;
  int DIVS; // Nr. of divisions to construct the bin
  int NR_BINS; // Nr of bins
  double DELTA; // size of the bins
  int DOF0;
  int DOF1;
};

#endif
