/* parameter.h */

#ifndef _parameter_h
#define _parameter_h

/*
 * Constants for Gravity vector(g)
 */
#define g1  -37.2
#define g2  -8.44
#define g3  1.02
#define g4  2.49e-1
#define g5  -2.82e-2

/*
 * Denavit-Hartenberg Parameters
 */
#define a2 0.4318
#define a3 -0.0203
#define d1 0.0
#define d2 0.2435
#define d3 -0.0934
#define d4 0.4331
#define d7 0.17       /* for EE in frame6 */
#define d23 0.1501   /* d23 = d2 + d3 */

/* 
 * Saturated Torques 
 */
#define T1max 97.6
#define T2max 156.4
#define T3max 89.4
#define T4max 24.2
#define T5max 20.1
#define T6max 21.3

/* 
 * parameters for singularity when arm is extended
 */
#define degree93 1.6176334 /* q3 = 92.6835689 deg */
#define degree88 1.530367 /* q3 = 87.6835689 deg */
#define degree83 1.4431005 /* q3 = 82.6835689 deg */
#define degree73 1.2685676 /* q3 = 72.6835689 deg */
#define degree113 1.9722221
#define dist93   0.8782965
#define dist88   0.8774850
#define dist83   0.8750521
#define dist73   0.8653459

/* 
 * some useful constants
 */
#define PI       3.1415927
#define pi2      6.2831853
#define pi_half  1.5707963
#define Deg2Rad  0.017453293  /* (pi/180.0) */
#define Rad2Deg 57.29578 /* (180/PI) */
#define degree5  0.0872665
#define degree10 0.1745329
#define degree20 0.34906585
#define cm3      0.03
#define cm5      0.05

/*
 * Macros for cosines and sines

#define c1 (c[0])
#define c2 (c[1])
#define c3 (c[2])
#define c4 (c[3])
#define c5 (c[4])
#define c6 (c[5])
#define s1 (s[0])
#define s2 (s[1])
#define s3 (s[2])
#define s4 (s[3])
#define s5 (s[4])
#define s6 (s[5])
#define c23 (c[6])
#define s23 (s[6])
*/

/*
 * Constants for Kinetic energy matrix(A)
 */
#define I1  1.43
#define I2  1.75
#define I3  1.38
#define I4  6.90e-1
#define I5  3.72e-1
#define I6  3.33e-1
#define I7  2.98e-1
#define I8  -1.34e-1
#define I9  2.38e-2
#define I10  -2.13e-2
#define I11  -1.42e-2
#define I12  -1.10e-2
#define I13  -3.79e-3
#define I14  1.64e-3
#define I15  1.25e-3
#define I16  1.24e-3
#define I17  6.42e-4
#define I18  4.31e-4
#define I19  3.00e-4
#define I20  -2.02e-4
#define I21  -1.00e-4
#define I22  -5.80e-5
#define I23  4.00e-4
#define Im1  1.14
#define Im2  4.71
#define Im3  8.27e-1
#define Im4  2.0e-1
#define Im5  1.79e-1
#define Im6  1.93e-1

/*
 * Other information
 */
/*
Base frame in universe
1.0 0.0 0.0   1.7844
0.0 1.0 0.0   0.7620
0.0 0.0 1.0   0.6731
Sensor frame with respect to wrist (for MW-10)
 0.0 -1.0  0.0   0.0000
 1.0  0.0  0.0   0.0000
 0.0  0.0  1.0   0.0850
Gripper frame with respect to wrist
 1.0 0.0 0.0   0.0000
 0.0 1.0 0.0   0.0000
 0.0 0.0 1.0   0.1700
Motor torque to voltage conversion factors
-0.09761
 0.05357
-0.11060
 0.41405
 0.47696
 0.46387
Center of gravity of end-effector in wrist frame in meters
0.0
0.0
0.1274
Mass of end-effector in kg
0.5280
*/

#endif /* parameter.h */



