#ifndef pumaivsimulator_h
#define pumaivsimulator_h

#include "baseivsimulator.h"
#include "dist.h"

class PumaIVSimulator : public BaseIVSimulator {
 public:

  PumaIVSimulator(SoSeparator *root_ = NULL);
  virtual ~PumaIVSimulator();

  // Inherited
  virtual bool IsClear(const Milestone& m);
  virtual bool IsClear(const Milestone& m0, const Milestone& m1, 
                       IntervalType itype=Closed);

  virtual double Cost(const GenCoords& p, const GenCoords& q);
  virtual double WspDist(const GenCoords& p, const GenCoords& q);
  virtual double NeighDist2(const GenCoords& p, const GenCoords& q);

  virtual bool IK(const Pose& x, GenCoords *q, const GenCoords *q0);
  virtual void FK(const GenCoords& q, Pose *x, int r =0);
  virtual void SetBase(const Pose& basePose_);
  virtual void InterpolatePath(const Path& p, Path *ip, double speed=1.0);

  // PumaIVSimulator specific
  bool IK(const Pose& x, GenCoords *q, int mode);

 protected:

  Frame3 toIV;
  Frame3 fromIV;
  Frame3 baseFrame;
  Frame3 ibaseFrame;

  double Distance(const GenCoords& p, const GenCoords& q);
  static void poseToFrame(Frame3 &frame, const Pose& pose);
  static void frameToPose(Pose& pose, const Frame3& frame);

 private:

  static double pmscale[6], pmaxtq[6];
  static Frame3 pbase[3];
  static int	fric;
  
  static void pmass(double m[6][6], double g[6], double q[6]);
  static void pdmodel(double m[6][6], double b[6][15], double c[6][6], 
		      double g[6], double q[6]);
  static void pdyn(double qdd[6], double q[6], double qd[6], double t[6]);
  static void pinvdyn(double t[6], double q[6], double qd[6], double qdd[6]);
  static void pcct(double t[6], double b[6][15], double c[6][6], double qd[6]);
  static void pkin(Frame3 f[7], const Frame3 *b, const double q[6]);
  static void pendeff(Frame3 *f, double q[6]);
  static void pjacobian(double J[6][6], double q[6]);
  static int  pinvkin(double q[6], const Frame3 *f, int mode);
  static double pke(double q[6], double qd[6]);
  static double ppe(double q[6]);
  static void pfric(double t[6], double qd[6]);
  static int  psim(double q[6], double qd[6], double tq[6], double t);

};

#endif








