
/////////////////////////////////////////////////////////////
//
// Source file for EditBoardUI
//
//    This class implements the user interface created in 
//    RapidApp.
//
//    Restrict changes to those sections between
//    the "//--- Start/End editable code block" markers
//
//    This will allow RapidApp to integrate changes more easily
//
//    This class is a ViewKit user interface "component".
//    For more information on how components are used, see the
//    "ViewKit Programmers' Manual", and the RapidApp
//    User's Guide.
//
//
/////////////////////////////////////////////////////////////


#include "EditBoardUI.h" // Generated header file for this class

#include <Xm/BulletinB.h> 
#include <Xm/List.h> 
#include <Xm/Scale.h> 
#include <Xm/ScrolledW.h> 
#include <Vk/VkResource.h>


// Externally defined classes referenced by this class: 

#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/So.h> // Includes ALL Inventor headers
                         // Replace for efficiency and faster compilation
//---- Start editable code block: headers and declarations


//---- End editable code block: headers and declarations



// These are default resources for widgets in objects of this class
// All resources will be prepended by *<name> at instantiation,
// where <name> is the name of the specific instance, as well as the
// name of the baseWidget. These are only defaults, and may be overriden
// in a resource file by providing a more specific resource name

String  EditBoardUI::_defaultEditBoardUIResources[] = {

        //---- Start editable code block: EditBoardUI Default Resources


        //---- End editable code block: EditBoardUI Default Resources

        (char*)NULL
};

EditBoardUI::EditBoardUI ( const char *name ) : VkComponent ( name ) 
{ 
    // No widgets are created by this constructor.
    // If an application creates a component using this constructor,
    // It must explictly call create at a later time.
    // This is mostly useful when adding pre-widget creation
    // code to a derived class constructor.

    //---- Start editable code block: EditBoard constructor 2


    //---- End editable code block: EditBoard constructor 2


}    // End Constructor




EditBoardUI::EditBoardUI ( const char *name, Widget parent ) : VkComponent ( name ) 
{ 
    //---- Start editable code block: EditBoard pre-create


    //---- End editable code block: EditBoard pre-create



    // Call creation function to build the widget tree.

     create ( parent );

    //---- Start editable code block: EditBoard constructor


    //---- End editable code block: EditBoard constructor


}    // End Constructor


EditBoardUI::~EditBoardUI() 
{
    delete _jointViewer;


    //---- Start editable code block: EditBoardUI destructor


    //---- End editable code block: EditBoardUI destructor
}    // End destructor



void EditBoardUI::create ( Widget parent )
{
    Arg      args[16];
    Cardinal count;
    count = 0;

    // Load any class-defaulted resources for this object

    setDefaultResources ( parent, _defaultEditBoardUIResources  );


    // Create an unmanaged widget as the top of the widget hierarchy

    _baseWidget = _editBoard = XtVaCreateWidget ( _name,
                                                  xmBulletinBoardWidgetClass,
                                                  parent, 
                                                  XmNresizePolicy, XmRESIZE_GROW, 
                                                  (XtPointer) NULL ); 

    // install a callback to guard against unexpected widget destruction

    installDestroyHandler();


    // Create widgets used in this component
    // All variables are data members of this class

    _dofNum = XtVaCreateManagedWidget  ( "dofNum",
                                          xmScaleWidgetClass,
                                          _baseWidget, 
                                          XmNorientation, XmHORIZONTAL, 
                                          XmNshowValue, True, 
                                          XmNx, 165, 
                                          XmNy, 25, 
                                          XmNwidth, 155, 
                                          XmNheight, 34, 
                                          (XtPointer) NULL ); 


    _dofScale = XtVaCreateManagedWidget  ( "dofScale",
                                            xmScaleWidgetClass,
                                            _baseWidget, 
                                            XmNorientation, XmHORIZONTAL, 
                                            XmNshowValue, True, 
                                            XmNdecimalPoints, 2, 
                                            XmNx, 165, 
                                            XmNy, 105, 
                                            XmNwidth, 155, 
                                            XmNheight, 35, 
                                            (XtPointer) NULL ); 

    XtAddCallback ( _dofScale,
                    XmNdragCallback,
                    &EditBoardUI::doSetDofCallback,
                    (XtPointer) this ); 


    _jointViewer = new SoXtExaminerViewer( _baseWidget, "jointViewer");
    _jointViewer->show();

    _jointViewer->setDecoration(False);


    _scrolledWindow1 = XtVaCreateManagedWidget  ( "scrolledWindow1",
                                                   xmScrolledWindowWidgetClass,
                                                   _baseWidget, 
                                                   XmNscrollBarDisplayPolicy, XmSTATIC, 
                                                   XmNx, 10, 
                                                   XmNy, 10, 
                                                   XmNwidth, 150, 
                                                   XmNheight, 300, 
                                                   (XtPointer) NULL ); 


    _jointList = XtVaCreateManagedWidget  ( "jointList",
                                             xmListWidgetClass,
                                             _scrolledWindow1, 
                                             XmNselectionPolicy, XmBROWSE_SELECT, 
                                             XmNwidth, 142, 
                                             XmNheight, 292, 
                                             (XtPointer) NULL ); 

    XtAddCallback ( _jointList,
                    XmNbrowseSelectionCallback,
                    &EditBoardUI::doSelectJointCallback,
                    (XtPointer) this ); 


    XtVaSetValues ( _jointViewer->getWidget(),
                    XmNx, 165, 
                    XmNy, 155, 
                    XmNwidth, 160, 
                    XmNheight, 160, 
                    (XtPointer) NULL );

    //---- Start editable code block: EditBoardUI create


    //---- End editable code block: EditBoardUI create
}

const char * EditBoardUI::className()
{
    return ("EditBoardUI");
}    // End className()




/////////////////////////////////////////////////////////////// 
// The following functions are static member functions used to 
// interface with Motif.
/////////////////////////////////// 

void EditBoardUI::doSelectJointCallback ( Widget    w,
                                          XtPointer clientData,
                                          XtPointer callData ) 
{ 
    EditBoardUI* obj = ( EditBoardUI * ) clientData;
    obj->doSelectJoint ( w, callData );
}

void EditBoardUI::doSetDofCallback ( Widget    w,
                                     XtPointer clientData,
                                     XtPointer callData ) 
{ 
    EditBoardUI* obj = ( EditBoardUI * ) clientData;
    obj->doSetDof ( w, callData );
}



/////////////////////////////////////////////////////////////// 
// The following functions are called from the menu items 
// in this window.
/////////////////////////////////// 

void EditBoardUI::doSelectJoint ( Widget, XtPointer ) 
{
    // This virtual function is called from doSelectJointCallback.
    // This function is normally overriden by a derived class.

}

void EditBoardUI::doSetDof ( Widget, XtPointer ) 
{
    // This virtual function is called from doSetDofCallback.
    // This function is normally overriden by a derived class.

}



//---- Start editable code block: End of generated code


//---- End editable code block: End of generated code
