/* ************************************************
     File ObstacleKit.C
     Enables the definition, creation and management
     of mobile obstacles in the workspace

     Gildardo Sanchez-Ante      
     Robotics Lab., Stanford University 
     ante@Robotics.stanford.edu
   ************************************************ */
#include "Obstacle.h"
#include "ObstacleKit.h"
#include "ColPrimitives.h"

#include <Inventor/SoPrimitiveVertex.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoShape.h>
#include <Inventor/actions/SoCallbackAction.h>
#include <Inventor/SoNodeKitPath.h>

#include <iostream.h>

SO_NODE_SOURCE(Obstacle);

void 
Obstacle::initClass() {
  SO_NODE_INIT_CLASS(Obstacle, SoSeparator, "Separator");
}

Obstacle::Obstacle() {
  SO_NODE_CONSTRUCTOR(Obstacle);

  colModel = NULL;
  oldObjModel = NULL;
  dist = 10e10;
  tol = 1;
}

Obstacle::~Obstacle() {
  delete colModel;
}

void 
Obstacle::buildColModel() 
{
  SoCallbackAction triAction;

  colModel = new PQP_Model;
  colModel->BeginModel();
  triAction.addTriangleCallback(SoShape::getClassTypeId(),
			      _TriangulateCB, colModel);
  triAction.apply(this);

  colModel->EndModel();
  initTransform();
  //  colModel->MemUsage(1);
}

PQP_Model *Obstacle::getColModel()
{
  return colModel;
}

void Obstacle::setTranslation(double x, double y, double z){
  trans[0]=x; trans[1]=y; trans[2]=z;
}

void Obstacle::initTransform() {
  for (int i = 0; i < 3; i++) { 
    for (int j = 0; j < 3; j++) {
      if(i==j) rot[i][j] = 1.0;
      else rot[i][j] = 0.0;
    }
    trans[i] = 0.0;
  }
}

void 
Obstacle::_TriangulateCB(void *data, SoCallbackAction *,
			  const SoPrimitiveVertex *vertex1,
			  const SoPrimitiveVertex *vertex2,
			  const SoPrimitiveVertex *vertex3) 
{
  static long cnt = 0;
  //  cout << cnt++ << endl;
  
  PQP_Model *colModel = (PQP_Model *)data;
  
  SbVec3f points[3];
  
  points[0] = vertex1->getPoint(); 
  points[1] = vertex2->getPoint(); 
  points[2] = vertex3->getPoint(); 
  
  if (colModel) {
    ColPoint3 verts[3];
    
    for (int i = 0; i < 3; i++)
      for (int j = 0; j < 3; j++)
	verts[i][j] = points[i][j];
    colModel->AddTri(verts[0], verts[1], verts[2], cnt++);
    //cerr<< cnt++<<endl;
  }
}









