#include "algo.h"
#include "Rn.h"
#include "GenCoords.h"

Rn::Rn() : randSrc_()
{
}


Rn::Rn(long seed) : randSrc_(seed)
{
}

/* UniformSample
 *   generate configurations uniformly at random in [0,1]^n
 */

void
Rn::UniformSample(GenCoords *q)
{
  int dim = GenCoords::Dim();
  const double *tmp = randSrc_.get(dim);

  memcpy(q->x, tmp, q->Dim()*sizeof(double));   // a little hacky, but fast
}


/* LocalUniformSample
 *   pick uniformly at random a configuraton in the neighborhood centered
 *   at q0 and with radius radius 
 *
 *     q0 : center of neighborhood
 * radius : size of the neighborhood 

 */

bool
Rn::LocalUniformSample(const GenCoords& q0, double radius, GenCoords *q)
{
  GenCoords& p =*q;

  int dim = GenCoords::Dim();
  const double *tmp = randSrc_.get(dim);
  //GenCoords nuevo(tmp);
  //nuevo.print();
  for (int i=0; i<dim; i++) {
    double nbhd[2];
    nbhd[0] = max( q0[i]-radius, 0.0 );
    nbhd[1] = min( q0[i]+radius, 1.0 );
    
    p[i] = nbhd[0] + tmp[i]*(nbhd[1]-nbhd[0]);
   } 
  return true;
}

/* LocalSample
 *   pick uniformly at random a configuraton 
 *   with distance smaller that radius to q0 
 */
bool
Rn::LocalSample(const GenCoords& q0, double radius, GenCoords *q)
{
  // This function gets
  double sqr_eps = radius*radius;
  double d =0 ;
  GenCoords& p =*q;
  int dim = GenCoords::Dim();
  do{
    const double *tmp = randSrc_.get(dim);
  
    for (int i=0; i<dim; i++) {
      double nbhd[2];
      nbhd[0] = max( q0[i]-radius, 0.0 );
      nbhd[1] = min( q0[i]+radius, 1.0 );
      
      p[i] = nbhd[0] + tmp[i]*(nbhd[1]-nbhd[0]);

      d += (q0[i]-p[i])*(q0[i]-p[i]);
    } 

  }  while(sqr_eps > d);
  return true;
}



