// //////////////////////////////////////////////////////////////
// //
// // Source file for SimMSainWindow
// //
// //    This class is a subclass of VkWindow
// //
// //
// // Normally, very little in this file should need to be changed.
// // Create/add/modify menus using RapidApp.
// //
// // Try to restrict any changes to the bodies of functions
// // corresponding to menu items, the constructor and destructor.
// //
// // Restrict changes to those sections between
// // the "//--- Start/End editable code block" markers
// //
// // Doing so will allow you to make changes using RapidApp
// // without losing any changes you may have made manually
// //
// // Avoid gratuitous reformatting and other changes that might
// // make it difficult to integrate changes made using RapidApp
// //
// //
// // Modified for having multiple robots 
// // Gildardo Sanchez-Ante
// // 
// //////////////////////////////////////////////////////////////

#include <stdio.h>
#include <string.h>
#include <iostream.h>

#include "Planner.h"
#include "stl_ext.h"

#include <Vk/VkApp.h>
#include <Vk/VkEZ.h>
#include <Vk/VkFileSelectionDialog.h>
#include <Vk/VkSubMenu.h>
#include <Vk/VkRadioSubMenu.h>
#include <Vk/VkMenuItem.h>
#include <Vk/VkMenuBar.h>
#include <Vk/VkResource.h>

#include <Xm/List.h>

#include <Inventor/nodes/SoCamera.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoBaseColor.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoVertexProperty.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoLineSet.h>
#include <Inventor/nodes/SoSwitch.h>
#include <Inventor/nodes/SoQuadMesh.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoBlinker.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/actions/SoWriteAction.h>

// Externally defined classes referenced by this class: 
#include "Hpaned.h"

extern void VkUnimplemented ( Widget, const char * );

//---- Start editable code block: headers and declarations
#include "Milestone.h"
#include "Rn.h"

#include "SimMainWindow.h"
#include "ivsimulator.h"
#include "EditMainWindow.h"
#include "ViewMainWindow.h"
#include "baseivsimulator.h"

//---- End editable code block: headers and declarations


// These are default resources for widgets in objects of this class
// All resources will be prepended by *<name> at instantiation,
// where <name> is the name of the specific instance, as well as the
// name of the baseWidget. These are only defaults, and may be overriden
// in a resource file by providing a more specific resource name

String  SimMainWindow::_defaultSimMainWindowResources[] = {
         (char*)NULL
};

//---- Class declaration

SimMainWindow::SimMainWindow (const char *name, IVSimulator *sim_) : 
                        VkWindow (name, NULL, 0)
{
    // Load any class-default resources for this object

    setDefaultResources ( baseWidget(), _defaultSimMainWindowResources  );


    // Create the view component contained by this window

    _hpaned = new Hpaned ( "hpaned",mainWindowWidget() );
    
    XtVaSetValues ( _hpaned->baseWidget(),
                    XmNwidth, 496, 
                    XmNheight, 512, 
                    (XtPointer) NULL );
    
    XtVaSetValues ( _baseWidget, XmNallowShellResize, False, NULL ); 
    
    // Add the component as the main view

    addView ( _hpaned );
    
    // Set up the window manager properties for this window.
    // This window is a coprimary window, as defined by the SGI Style Guide
    
     VkConfigureWM ( this,
                    FALSE,   //quit
                    TRUE,  // close
                    TRUE,   // border
                    TRUE,   // title
                    TRUE,   // allow resize
                    TRUE,   // allow iconify
                    TRUE);  // Show window manager menu
    
    // Create the panes of this window's menubar. The menubar itself
    // is created automatically by ViewKit


    // The filePane menu pane

    _filePane =  addMenuPane ( "File" );

    _openRobotButton =  _filePane->addAction ( "Load Robot", 
				   &SimMainWindow::openRobotCallback, 
				   (XtPointer) this );

    _openWorkspaceButton =  _filePane->addAction ( "Load Workspace", 
			    &SimMainWindow::openWorkspaceCallback, 
				   (XtPointer) this );

    _separator1 =  _filePane->addSeparator();


    _exitButton =  _filePane->addAction ( "Quit", 
                                        &SimMainWindow::quitCallback, 
                                        (XtPointer) this );

    // The viewPane menu pane

    _viewPane =  addRadioMenuPane("Robot");

    _editModeToggle =  _viewPane->addToggle ( "Modify Configuration", 
                                        &SimMainWindow::setEditModeCallback, 
                                        (XtPointer) this );

    _viewModeToggle =  _viewPane->addToggle ( "Animation Control", 
                                        &SimMainWindow::setViewModeCallback, 
                                        (XtPointer) this );

    _noModeToggle =  _viewPane->addToggle ( "No Mode", 
                                        &SimMainWindow::setNoModeCallback, 
                                        (XtPointer) this );
    _noModeToggle->setVisualState(TRUE);

    // The editPane menu pane

    _demosPane =  addMenuPane ( "Demos" );
    
    _setDemo1Button =  _demosPane->addAction ( "A simple environment", 
                                        &SimMainWindow::setDemo1Callback, 
                                        (XtPointer) this );
    _setDemo2Button =  _demosPane->addAction ( "Moving between some spheres", 
                                        &SimMainWindow::setDemo2Callback, 
                                        (XtPointer) this );
    _setDemo3Button =  _demosPane->addAction ( "Moving in an Office", 
                                        &SimMainWindow::setDemo3Callback, 
                                        (XtPointer) this );
    _setDemo4Button =  _demosPane->addAction ( "Narrow Passage", 
                                        &SimMainWindow::setDemo4Callback, 
                                        (XtPointer) this );
    _setDemo5Button =  _demosPane->addAction ( "Hole in the wall", 
                                        &SimMainWindow::setDemo5Callback, 
                                        (XtPointer) this );
    _setDemo6Button =  _demosPane->addAction ( "Manufacturing Cell", 
                                        &SimMainWindow::setDemo6Callback, 
                                        (XtPointer) this );
    _setDemo7Button =  _demosPane->addAction ( "Welding a Car", 
                                        &SimMainWindow::setDemo7Callback, 
					(XtPointer) this );  

    // The operationsPane menu pane

    _operationsPane =  addMenuPane ( "Operations" );

    _setRobotsButton =  _operationsPane->addAction ( "Set Init Robots Pose", 
				       &SimMainWindow::setRobotsCallback, 
                                       (XtPointer) this );

    _separator2 =  _operationsPane->addSeparator();

    _colCheckButton =  _operationsPane->addAction ( "Check for Collision", 
                                        &SimMainWindow::colCheckCallback, 
                                        (XtPointer) this );

    _distCompButton =  _operationsPane->addAction ( "Compute distace", 
                                        &SimMainWindow::distCompCallback, 
                                        (XtPointer) this );

    _separator3 =  _operationsPane->addSeparator();

    _setInitConfButton =  _operationsPane->addAction ( "Set Initial Config", 
                                     &SimMainWindow::setInitConfCallback, 
                                        (XtPointer) this );

    _setGoalConfButton =  _operationsPane->addAction ( "Set Goal Config", 
                                        &SimMainWindow::setGoalConfCallback, 
                                        (XtPointer) this );

    _separator4 =  _operationsPane->addSeparator();

    _runPlannerButton =  _operationsPane->addAction ( "Run Planner", 
                                        &SimMainWindow::runPlannerCallback, 
                                        (XtPointer) this );

    _animatePathButton =  _operationsPane->addAction ( "Load Path for Animation", 
                                        &SimMainWindow::animatePathCallback, 
                                        (XtPointer) this );

    //---- Start editable code block: SimMainWindow constructor
    
    IVSimulator::initInventor();
    
    root = (SoSeparator *)viewer()->getSceneGraph();
    root->addChild(createFloor(10, 10));


    // The if() is for the SGI Version
    if (sim_ != NULL) {
      sim = sim_;
      sim->setRoot(root);
    } else 
      sim = new IVSimulator(root);
    
    /* And this is for LINUX
       sim = new PumaIVSimulator(root); 
       sim->setRoot(root);
    */

    for(int k=0;k<ROBOTS;k++)
      {
	editWindow[k] = new EditMainWindow("");
	editWindow[k]->setRob(k);
	editWindow[k]->setSim(sim);
	editWindow[k]->setCamera(viewer()->getCamera());
      }
    
    viewWindow = new ViewMainWindow("");
    viewWindow->setSim(sim); 
    //---- End editable code block: SimMainWindow constructor
    
}    // End Constructor


SimMainWindow::~SimMainWindow()
{
    delete _hpaned;
    //---- Start editable code block: SimMainWindow destructor


    //---- End editable code block: SimMainWindow destructor
}    // End destructor


const char *SimMainWindow::className()
{
    return ("SimMainWindow");
}    // End className()


Boolean SimMainWindow::okToQuit()
{
    //---- Start editable code block: SimMainWindow okToQuit

    // This member function is called when the user quits by calling
    // theApplication->terminate() or uses the window manager close protocol
    // This function can abort the operation by returning FALSE, or do some.
    // cleanup before returning TRUE. The actual decision is normally passed on
    // to the view object


    // The view object is an Inventor component, which does
    // not currently support the okToQuit protocol

    return ( TRUE );
    //---- End editable code block: SimMainWindow okToQuit
}    // End okToQuit()



/////////////////////////////////////////////////////////////// 
// The following functions are static member functions used to 
// interface with Motif.
/////////////////////////////////// 

void SimMainWindow::animatePathCallback ( Widget    w,
				       XtPointer clientData,
				       XtPointer callData ) 
{ 
  SimMainWindow* obj = ( SimMainWindow * ) clientData;
  obj->doAnimatePath ( w, callData );
}

void SimMainWindow::colCheckCallback ( Widget    w,
				       XtPointer clientData,
				       XtPointer callData ) 
{ 
    SimMainWindow* obj = ( SimMainWindow * ) clientData;
    obj->doColCheck ( w, callData );
}

void SimMainWindow::distCompCallback ( Widget    w,
                                         XtPointer clientData,
                                         XtPointer callData ) 
{ 
  SimMainWindow* obj = ( SimMainWindow * ) clientData;
  obj->doDistComp ( w, callData );
}

void SimMainWindow::runPlannerCallback ( Widget    w,
                                         XtPointer clientData,
                                         XtPointer callData ) 
{ 
  SimMainWindow* obj = ( SimMainWindow * ) clientData;
  obj->doRunPlanner ( w, callData );
}

void SimMainWindow::openWorkspaceCallback ( Widget    w,
                                               XtPointer clientData,
                                               XtPointer callData ) 
{ 
    SimMainWindow* obj = ( SimMainWindow * ) clientData;
    obj->doOpenWorkspace( w, callData );
}
void SimMainWindow::setDemo1Callback ( Widget    w,
				       XtPointer clientData,
				       XtPointer callData ) 
{ 
  SimMainWindow* obj = ( SimMainWindow * ) clientData;
  obj->doSetDemo1 ( w, callData );
}

void SimMainWindow::setDemo2Callback ( Widget    w,
				       XtPointer clientData,
				       XtPointer callData ) 
{ 
  SimMainWindow* obj = ( SimMainWindow * ) clientData;
  obj->doSetDemo2 ( w, callData );
}

void SimMainWindow::setDemo3Callback ( Widget    w,
				       XtPointer clientData,
				       XtPointer callData ) 
{ 
  SimMainWindow* obj = ( SimMainWindow * ) clientData;
  obj->doSetDemo3 ( w, callData );
}

void SimMainWindow::setDemo4Callback ( Widget    w,
				       XtPointer clientData,
				       XtPointer callData ) 
{ 
  SimMainWindow* obj = ( SimMainWindow * ) clientData;
  obj->doSetDemo4 ( w, callData );
}
void SimMainWindow::setDemo5Callback ( Widget    w,
				       XtPointer clientData,
				       XtPointer callData ) 
{ 
  SimMainWindow* obj = ( SimMainWindow * ) clientData;
  obj->doSetDemo5 ( w, callData );
}
void SimMainWindow::setDemo6Callback ( Widget    w,
				       XtPointer clientData,
				       XtPointer callData ) 
{ 
  SimMainWindow* obj = ( SimMainWindow * ) clientData;
  obj->doSetDemo6 ( w, callData );
}
void SimMainWindow::setDemo7Callback ( Widget    w,
				       XtPointer clientData,
				       XtPointer callData ) 
{ 
  SimMainWindow* obj = ( SimMainWindow * ) clientData;
  obj->doSetDemo7 ( w, callData );
}

void SimMainWindow::setGoalConfCallback ( Widget    w,
					  XtPointer clientData,
					  XtPointer callData ) 
{ 
  SimMainWindow* obj = ( SimMainWindow * ) clientData;
  obj->doSetGoalConf ( w, callData );
}

void SimMainWindow::setInitConfCallback ( Widget    w,
					  XtPointer clientData,
					  XtPointer callData ) 
{ 
  SimMainWindow* obj = ( SimMainWindow * ) clientData;
  obj->doSetInitConf ( w, callData );
}

void SimMainWindow::setRobotsCallback ( Widget    w,
					XtPointer clientData,
					XtPointer callData ) 
{ 
  SimMainWindow* obj = ( SimMainWindow * ) clientData;
  obj->doSetRobots ( w, callData );
}

void SimMainWindow::openRobotCallback ( Widget    w,
					XtPointer clientData,
					XtPointer callData ) 
{ 
  SimMainWindow* obj = ( SimMainWindow * ) clientData;
  obj->doOpenRobot ( w, callData );
}

void SimMainWindow::quitCallback ( Widget    w,
                                   XtPointer clientData,
                                   XtPointer callData ) 
{ 
  SimMainWindow* obj = ( SimMainWindow * ) clientData;
  obj->doQuit ( w, callData );
}


void SimMainWindow::setEditModeCallback ( Widget    w,
                                          XtPointer clientData,
                                          XtPointer callData ) 
{ 
  SimMainWindow* obj = ( SimMainWindow * ) clientData;
  obj->doSetEditMode ( w, callData );
}


void SimMainWindow::setNoModeCallback ( Widget    w,
                                        XtPointer clientData,
                                        XtPointer callData ) 
{ 
  SimMainWindow* obj = ( SimMainWindow * ) clientData;
  obj->doSetNoMode ( w, callData );
}

void SimMainWindow::setViewModeCallback ( Widget    w,
                                          XtPointer clientData,
                                          XtPointer callData ) 
{ 
    SimMainWindow* obj = ( SimMainWindow * ) clientData;
    obj->doSetViewMode ( w, callData );
}


/////////////////////////////////////////////////////////////// 
// The following functions are called from callbacks 
/////////////////////////////////// 

void SimMainWindow::doAnimatePath ( Widget w, XtPointer callData ) 
{
  // This function sends the path extracted from the tree to
  // a function that gets an interpolated path, which is sent
  // to the animation module

  // Here we will read and draw the non-optimal path 
  FILE *fPathFile;
  Milestone m; 
  float q[6];

  fPathFile = fopen("freePath.txt","rt");
  if (fPathFile==NULL){
    cerr<<"Error in SimMainWindow::doAnimatePath when opening file"<<endl;
    return ;
  }

  while(!feof(fPathFile)){
    fscanf(fPathFile, "%f  %f  %f  %f  %f  %f\n",
	   &q[0],&q[1],&q[2],&q[3],&q[4],&q[5]);
    GenCoords gq(q);
    m.setConfig(gq);
    freePath[0].push_back(m);
  };
  fclose(fPathFile);

  // The first path found... in red on the screen
  Draw(freePath[0],0);

  // Here we draw the path that was optimized
  // printed in yellow
  Draw(optimalPath[0],0);

}    // End SimMainWindow::doLoadPath()


void SimMainWindow::doColCheck ( Widget w, XtPointer callData ) 
{
  // Check if the Robot in the current config is in collision
  GenCoords q;
  sim->GetConfig(&q,0);
  Milestone r;
  r.setConfig(q);
 
  r.print();
  cout << "Currently " << (sim->IsClear(r) ? "\n" : "\nnot ") << "clear" << endl;
  
}    // End SimMainWindow::doColCheck()


void SimMainWindow::doDistComp ( Widget w, XtPointer callData ) 
{
  // Computes the shortest distance from the robot to the obstacles
  GenCoords q;
  sim->GetConfig(&q,0);
  sim->printConfig(q);

  cout << "Current distance: " << sim->getCurDist(0) << endl;

}    // End SimMainWindow::doDistComp()


void SimMainWindow::doRunPlanner ( Widget w, XtPointer callData ) 
{
  // This function assumes that the initial and final configurations
  // have been already defined...
  GenCoords q;
  Pose p, basePose;
 
  //Creates a planner
  Planner myPlanner(sim);

  // Set the robots in their positions
  sim->GetConfig(&q,0);
  sim->FK(q, &p);

  ((BaseIVSimulator *)sim)->SetBase(basePose);
    
  myPlanner.Query(q_init,q_goal,freePath[0],0.15);
}    // End SimMainWindow::doFindPath()


void SimMainWindow::doOpenWorkspace( Widget w, XtPointer callData ) 
{
  //---- Start editable code block: SimMainWindow doOpenWorkButton
  if(theFileSelectionDialog->postAndWait() == VkDialogManager::OK)
    {
      const char *filename = theFileSelectionDialog->fileName();
      openWorkspaceFile(filename);
    }
  //---- End editable code block: SimMainWindow doOpenWorkButton
}    // End SimMainWindow::doOpenWorkButton()


void SimMainWindow::doSetGoalConf ( Widget w, XtPointer callData ) 
{
  //---- Start editable code block: SimMainWindow doSetFinalPath

  sim->GetConfig(&q_goal,0);

  //---- End editable code block: SimMainWindow doSetFinalPath
}    // End SimMainWindow::doSetFinalPath()


void SimMainWindow::doSetInitConf ( Widget w, XtPointer callData ) 
{
  //---- Start editable code block: SimMainWindow doSetInitialConf
  sim->GetConfig(&q_init,0);

  //---- End editable code block: SimMainWindow doSetInitialConf
}    // End SimMainWindow::doSetInitialConf()


void SimMainWindow::doSetRobots ( Widget w, XtPointer callData ) 
{
  //---- Start editable code block: SimMainWindow doSetRobots
  GenCoords q;
  Pose p, basePose;
  
  sim->GetConfig(&q,0);
  ((PumaIVSimulator *)sim)->FK(q, &p);
  
  ((BaseIVSimulator *)sim)->SetBase(basePose);
}    // End SimMainWindow::doSetRobots()

void SimMainWindow::doOpenRobot ( Widget w, XtPointer callData ) 
{
  if(theFileSelectionDialog->postAndWait() == VkDialogManager::OK)
    {
      const char *filename = theFileSelectionDialog->fileName();
      openRobotFile(filename);
    }
}    // End SimMainWindow::openRobotFile()


void SimMainWindow::doQuit ( Widget, XtPointer ) 
{
  // Exit via quitYourself() to allow the application
  // to go through its normal shutdown routines, checking with
  // all windows, views, and so on.
  theApplication->quitYourself();
}    // End SimMainWindow::quit()


void SimMainWindow::doSetEditMode ( Widget w, XtPointer callData ) 
{
  //---- Start editable code block: SimMainWindow setEditMode
  if ((int)EZ(w))
    editWindow[0]->show();
  else 
    editWindow[0]->hide();
  //---- End editable code block: SimMainWindow setEditMode
}    // End SimMainWindow::


void SimMainWindow::doSetNoMode ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: SimMainWindow setNoMode

    //---- End editable code block: SimMainWindow setNoMode

}    // End SimMainWindow::setNoMode()

/* *********************************************************
           START OF DEFINITION OF TEST ENVIRONMENTS
   *********************************************************
*/

void SimMainWindow::doSetDemo1 ( Widget w, XtPointer callData ) 
{
  // Cluttered environment
  GenCoords q;
  Pose p, basePose;
  Milestone m;
  
  double final[]={0.27, 0.80, 0.59, 0.81, 0.38, 0.46},
       initial[]={0.61, 0.78, 0.58, 0.81, 0.38, 0.12};
	   
  GenCoords start(initial), goal(final);

  openRobotFile("../models/puma/pumaiv.bak");
  openWorkspaceFile("../models/Env/ClutteredEnv.iv");

  
  Planner myPlanner(sim);

  sim->GetConfig(&q,0);
  sim->FK(q, &p);
  ((BaseIVSimulator *)sim)->SetBase(basePose);
 
  myPlanner.Query(start,goal,optimalPath[0],0.15);
  
}    // End SimMainWindow::doSetDemo1()


void SimMainWindow::doSetDemo2 ( Widget w, XtPointer callData ) 
{
  // Moving between several spheres
  GenCoords q;
  Pose p, basePose;
  Milestone m;
  
  double initial[]= {0.22, 0.94, 0.63, 0.51, 0.68, 0.79},
	   final[]  = {0.90, 0.64, 0.62, 0.45, 0.72, 0.75};
	 
		      
  GenCoords start(initial), goal(final);

  openRobotFile("../models/puma/pumaiv.bak");
  openWorkspaceFile("../models/Env/SpheresEnv.iv");
 
  Planner myPlanner(sim);
  
  sim->GetConfig(&q,0);
  sim->FK(q, &p);
  ((BaseIVSimulator *)sim)->SetBase(basePose);
  
  myPlanner.Query(start,goal,optimalPath[0],0.15);
}    // End SimMainWindow::doSetDemo2()


void SimMainWindow::doSetDemo3 ( Widget w, XtPointer callData ) 
{
  // Moving in an office
  GenCoords q;
  Pose p, basePose;
  Milestone m;
  
  double initial[]={0.68, 0.76, 0.66, 0.50, 0.70, 0.81},
	   final[]={0.44, 0.62, 0.87, 0.50, 0.80, 0.15};
	   
  GenCoords start(initial), goal(final);

  openRobotFile("../models/puma/pumaiv.bak");
  openWorkspaceFile("../models/Env/DesktopEnv.iv");

  Planner myPlanner(sim);
  
  sim->GetConfig(&q,0);
  sim->FK(q, &p);
  ((BaseIVSimulator *)sim)->SetBase(basePose);
  
  myPlanner.Query(start,goal,optimalPath[0],0.15);
}    // End SimMainWindow::doSetDemo3()

void SimMainWindow::doSetDemo4 ( Widget w, XtPointer callData ) 
{
  // Narrow Passage
  GenCoords q;
  Pose p, basePose;
  Milestone m;
  
  double initial[]={0.86, 0.87, 0.52, 0.19, 0.54, 0.81},
	   final[]={0.02, 0.96, 0.11, 0.81, 0.10, 0.75};
	   
  GenCoords start(initial), goal(final);

  openRobotFile("../models/puma/pumaiv.bak");
  openWorkspaceFile("../models/Env/NarrowEnv.iv");

  Planner myPlanner(sim);
  
  sim->GetConfig(&q,0);
  sim->FK(q, &p);
  ((BaseIVSimulator *)sim)->SetBase(basePose);
  
  myPlanner.Query(start,goal,optimalPath[0],0.15);
}    // End SimMainWindow::doSetDemo4()

void SimMainWindow::doSetDemo5 ( Widget w, XtPointer callData ) 
{
  // Through the hole
  GenCoords q;
  Pose p, basePose;
  Milestone m;
  
  double initial[]={0.77, 0.84, 0.79, 0.50, 0.50, 0.50},
	   final[]={0.23, 0.75, 0.57, 0.50, 0.50, 0.50};
	   
  GenCoords start(initial), goal(final);

  openRobotFile("../models/puma/pumaiv.bak");
  openWorkspaceFile("../models/Env/HoleEnv.iv");

  Planner myPlanner(sim);
  
  sim->GetConfig(&q,0);
  sim->FK(q, &p);
  ((BaseIVSimulator *)sim)->SetBase(basePose);
  
  myPlanner.Query(start,goal,optimalPath[0],0.15);
}    // End SimMainWindow::doSetDemo5()

void SimMainWindow::doSetDemo6 ( Widget w, XtPointer callData ) 
{
  // Manufacturing Cell
  GenCoords q;
  Pose p, basePose;
  Milestone m;
  
  double initial[]={0.42, 0.64, 0.78, 0.46, 0.94, 0.12},
	   final[]={0.82, 0.73, 0.73, 0.52, 0.81, 0.50};
	   
  GenCoords start(initial), goal(final);

  openRobotFile("../models/puma/pumaLathe.iv");
  openWorkspaceFile("../models/Env/ManCellEnv.iv");

  Planner myPlanner(sim);
  
  sim->GetConfig(&q,0);
  sim->FK(q, &p);
  ((BaseIVSimulator *)sim)->SetBase(basePose);
  
  myPlanner.Query(start,goal,optimalPath[0],0.15);
}

void SimMainWindow::doSetDemo7 ( Widget w, XtPointer callData ) 
{
  
  // welding a car
  GenCoords q;
  Pose p, basePose;
  Milestone m;
  double initial[]={0.61,0.78,0.58, 0.81, 0.38, 0.12},
	   final[]={0.27, 0.80, 0.59, 0.81, 0.38, 0.46};
	   
  GenCoords start(initial), goal(final);
  openRobotFile("../models/puma/pumaiv.bak");
  openWorkspaceFile("../models/Env/WeldEnv.iv");
  

  Planner myPlanner(sim);
  
  sim->GetConfig(&q,0);
  sim->FK(q, &p);
  ((BaseIVSimulator *)sim)->SetBase(basePose);
  
  myPlanner.Query(start,goal,optimalPath[0],0.15);

}    // End SimMainWin:Demo7()

/* *********************************************************
           END OF DEFINITION OF TEST ENVIRONMENTS
   *********************************************************
*/


void SimMainWindow::doSetViewMode ( Widget w, XtPointer callData ) 
{
  //---- Start editable code block: SimMainWindow setViewMode
  
  if ((int)EZ(w)) {
    if (viewWindow->toggleAnimation(true)) {
      viewWindow->show();
    }
  }
  else {
    viewWindow->toggleAnimation(false);
    viewWindow->hide();
  }
  //---- End editable code block: SimMainWindow setViewMode
  
}    // End SimMainWindow::setViewMode()


//---- Start editable code block: End of generated code

SoXtExaminerViewer *SimMainWindow::viewer() {return _hpaned->viewer();}

bool SimMainWindow::openRobotFile(const char *filename) {
  if (!sim->readRobot(filename))
    return false;
  
  for(int k=0;k<ROBOTS;k++)
    editWindow[k]->resetSim();
  return true;
}

bool SimMainWindow::openWorkspaceFile(const char *filename) {
  SbVec3f p;

  if (!sim->readWorkspace(filename))
    return false;
  else
    return true;
}

IVSimulator *SimMainWindow::getSimulator() {
	return sim;
}

void SimMainWindow::setSimulator(IVSimulator *sim_) {
  sim = sim_;
  sim->setRoot(root);

  for(int k=0;k<ROBOTS;k++)
    {
      editWindow[k]->setSim(sim);editWindow[k]->setRob(k);
      editWindow[k]->setCamera(viewer()->getCamera());
    }
  viewWindow->setSim(sim);
}

void SimMainWindow::Draw(const Path& p, int n) {
  Path ip; 

  SbViewVolume vol=viewer()->getCamera()->getViewVolume();
  float dx=vol.getWidth(), dy=vol.getHeight();
  float dl = sqrt(dx*dx+dy*dy);
   
  // p is the path that was obtained by the planner (could be a simple
  // one or an quasi-optimal, ip is the interpolated path
  sim->InterpolatePath(p, &ip, 1.0);
 
  list<Milestone>::const_iterator mi;
  Milestone v;
  GenCoords q;

  FILE *fp;
  fp=fopen("interpo.txt","wt");

  if (fp==NULL){
    cerr<<"Error in Planner::write_path when opening file"<<endl;
    return ;
  }
  
  for ( mi = ip.begin();
	mi != ip.end();
	mi++)
    { 
      v = (*mi);
      q=v.getConfig();
      fprintf(fp, "%f  %f  %f  %f  %f  %f  %f\n",
	      q[0],q[1],q[2],q[3],q[4],q[5],v.getTime());
    }
  fclose(fp);

  float markersize = 0.1*dl;
  
  SoSwitch *wp = getWristPath(p, ip,markersize);

  viewWindow->setPath(ip, wp);
}

/* FOR_EACH
 *   a macro to iterate through a container
 */

#ifndef ForEach
#define ForEach(iter, container) \
    for(iter=(container).begin(); iter != (container).end(); iter++)
#endif

SoSwitch*
SimMainWindow::getWristPath(const Path& p, const Path& ip, float scale,int r)
{
  // parent has two children. 
  // First child is wristpath, trajectory for the interpolated path.
  // Second child is markerpath, a path of  markers.
  SoSwitch* parent = new SoSwitch;
 
  SoSeparator* markerpath = new SoSeparator;
  SoSeparator* marker = createMarker(mrkFrame, scale);

  Path::const_iterator pi;
  Pose pose;

  // mark milestones
  ForEach (pi, p) {
    SoSeparator* posedmarker = new SoSeparator;
   
    SoTransform *xform = new SoTransform;
    GenCoords q;
    q = (*pi).getConfig();
    sim->FK( q, &pose);
    xform->setMatrix(pose);
    posedmarker->addChild(xform);
    posedmarker->addChild(marker);
    markerpath->addChild(posedmarker);
  }

  // wrist-point trajectory
  SoVertexProperty *vts = new SoVertexProperty; 
  vts->ref(); //commented
  int i=0;
  ForEach (pi, ip) {
    GenCoords q;
    q = (*pi).getConfig();
 
    sim->FK( q, &pose);
    SoTransform *xform = new SoTransform;
 
    xform->setMatrix(pose);
    vts->vertex.set1Value(i++, xform->translation.getValue() );
    xform->unref();
  }

  SoSeparator *wristpath = new SoSeparator;

  SoDrawStyle *myDrawStyle = new SoDrawStyle;
  myDrawStyle->lineWidth.setValue(2);

  SoLineSet *traj = new SoLineSet;
  traj->numVertices.set1Value(0, int(ip.size()) );
  traj->vertexProperty = vts;

  SoSwitch *colorSwitch = new SoSwitch;
  SoBaseColor *myColor1 = new SoBaseColor; 
  myColor1->rgb.setValue(0.8, 0.8, 0.1); // color for current path
  SoBaseColor *myColor2 = new SoBaseColor; 
  myColor2->rgb.setValue(0.8, 0.1, 0.1); // color for old path

  colorSwitch->addChild(myColor2);
  colorSwitch->addChild(myColor1);
  colorSwitch->whichChild = SO_SWITCH_INHERIT;

  wristpath->addChild(myDrawStyle);
  wristpath->addChild(colorSwitch);
  wristpath->addChild(traj);

  parent->addChild(wristpath);
  parent->addChild(markerpath);// commented

  return parent;
}

/* createMarker
 *    mtype : type of marker
 *    scale : size of marker
 */
SoSeparator*
SimMainWindow::createMarker(int mtype, float scale)
{
  SoSeparator* marker;

  switch (mtype) {

  case mrkFrame:
    marker = createFrame(scale);
    break;
 
  case mrkBall:
    marker = createBall(scale);
    break;

  default:
    marker = 0;
  }

  return marker;
}

/* createBall
 *   create a unit ball
 */

SoSeparator*
SimMainWindow::createBall(float scale)
{
  SoSeparator* parent = new SoSeparator;

  // create a ball at the origin
  SoMaterial* myMaterial = new SoMaterial;
  myMaterial->diffuseColor.setValue(0.6, 0.6, 0.0);
  parent->addChild(myMaterial);

  SoSphere* myBall = new SoSphere;
  myBall->radius = scale;
  parent->addChild(myBall);

  return parent;
}


/* createFrame
 *   create a coord sys object of unit size
 */
SoSeparator*
SimMainWindow::createFrame(float scale)
{
  SoSeparator* frame = new SoSeparator;

    // create the cylinder
  SoCylinder* myCyl = new SoCylinder;
  myCyl->height = 0.9 * scale;
  myCyl->radius = 0.04 * scale ;      /* the size of the axis the inversely
					 proportional to scale */
  myCyl->parts = SoCylinder::SIDES;
    
  // create the arrow of the axis
  SoCone* myCone = new SoCone;
  myCone->bottomRadius = 0.08 * scale;
  myCone->height = 0.1 * scale;

  // make an axis
  SoSeparator* axis = new SoSeparator;
  SoTransform* xform = new SoTransform;
  xform->translation.setValue(.0, 0.45 * scale ,.0);
  axis->addChild( xform );
  axis->addChild( myCyl );

  xform = new SoTransform;
  xform->translation.setValue(.0, .5 * scale , .0);
  axis->addChild( xform );
  axis->addChild( myCone);

  // make x-axis
  SoSeparator* xx = new SoSeparator;
  SoMaterial* myMaterial = new SoMaterial;
  myMaterial->diffuseColor.setValue(.2, 1.0, 0.2);
  xx->addChild( myMaterial );

  SoRotationXYZ* myRot = new SoRotationXYZ;
  myRot->axis = SoRotationXYZ::Z;
  myRot->angle = -M_PI/2;
  xx->addChild( myRot );

  xx->addChild( axis );

  // make y-axis
  SoSeparator* yy = new SoSeparator;
  myMaterial = new SoMaterial;
  myMaterial->diffuseColor.setValue(.2, 1.0, 0.2);
  yy->addChild( myMaterial );

  yy->addChild( axis );

  // make z-axis
  SoSeparator* zz = new SoSeparator;
  myMaterial = new SoMaterial;
  myMaterial->diffuseColor.setValue(.2, 1.0, 0.2);
  zz->addChild( myMaterial );

  myRot = new SoRotationXYZ;
  myRot->axis = SoRotationXYZ::X;
  myRot->angle = M_PI/2;
  zz->addChild( myRot );

  zz->addChild( axis );

  // finally add all axes to the frame
  frame->addChild( xx);
  frame->addChild( yy);
  frame->addChild( zz);

  // create a ball at the origin
  SoSeparator* origin = new SoSeparator;
  myMaterial = new SoMaterial;
  myMaterial->diffuseColor.setValue(0.4, 0.4, 0.4);
  SoSphere* myBall = new SoSphere;
  myBall->radius = 0.08 * scale;
  origin->addChild(myMaterial);
  origin->addChild(myBall);

  frame->addChild( origin);
    
  return frame;
}

SoSeparator* 
SimMainWindow::createFloor(float size, int density) {
  double dx = 10;                       // carpet size 
  double dz = 10;
  double dy = 0.02;                     // carpet-grid separation
  int gridsize = 10;                    // grid size

  SoSeparator *parent = new SoSeparator; 

  // carpet
  SoBaseColor *mycolor = new SoBaseColor; 
  mycolor->rgb.setValue(0.2, 0.7, 0.7);
  parent->addChild(mycolor);

  SoVertexProperty *vts = new SoVertexProperty; 
  vts->vertex.set1Value(0, -0.5*dx, -1.5*dy,  0.5*dz);
  vts->vertex.set1Value(1, -0.5*dx, -1.5*dy, -0.5*dz);
  vts->vertex.set1Value(2,  0.5*dx, -1.5*dy,  0.5*dz);
  vts->vertex.set1Value(3,  0.5*dx, -1.5*dy, -0.5*dz);

  SoQuadMesh *carpet = new SoQuadMesh;
  carpet->verticesPerColumn = 2;
  carpet->verticesPerRow = 2;
  carpet->vertexProperty = vts;
  parent->addChild(carpet);

  // gridlines
  mycolor = new SoBaseColor; 
  
  mycolor->rgb.setValue(0.1, 0.1 ,0.1);
  parent->addChild(mycolor);
	SoLineSet *grid = new SoLineSet;
  vts = new SoVertexProperty; 
  grid->vertexProperty = vts;

  int vi=0, l=0;

  // vertical lines
  int i;
  for (i=1; i<gridsize; i++) {
    float h=-0.5*dx + float(i) / gridsize * dx;
    vts->vertex.set1Value(vi++, h, -dy, -0.5*dz);
    vts->vertex.set1Value(vi++, h, -dy,  0.5*dz);
    grid->numVertices.set1Value(l++, 2);
  }

  // horizontal lines
  for (i=1; i<gridsize; i++) {
    float v=-0.5*dz + float(i) / gridsize * dz;
    vts->vertex.set1Value(vi++, -0.5*dx, -dy, v );
    vts->vertex.set1Value(vi++,  0.5*dx, -dy, v );
    grid->numVertices.set1Value(l++, 2);
  }
  parent->addChild(vts);

  parent->addChild(grid);

  return parent;

}
SoSeparator* 
SimMainWindow::getDecorations()
{
  SoSeparator *parent = new SoSeparator;
  SoSeparator *child;

  //if (child = createFloor()) parent->addChild( child  );
  //if (child = createSky())   parent->addChild( child  );

  //if (child = createDecor() ) parent->addChild( child );
  //sim->readD(parent);
  return parent;
}

SoSeparator*
SimMainWindow::createSky()
{
  SoInput sceneinput;
  if (!sceneinput.openFile("sky.iv")) {
    cerr << "SimMainWindow::createSky : cannot open sky.iv" << endl;
    return 0;
  }

  SoSeparator *sky = SoDB::readAll(&sceneinput);
  if (!sky)
    cerr << "SimMainWindow::createSky : failed in reading sky.iv" << endl;

  sceneinput.closeFile();

  return sky;
}

SoSeparator*
SimMainWindow::createDecor()
{
  SoInput sceneinput;
  if (!sceneinput.openFile("decorations.iv")) {
    cerr << "SimMainWindow::ceateDecor : cannot open decorations.iv" << endl;
    return 0;
  }

  SoSeparator *decor = SoDB::readAll(&sceneinput);
  if (!decor)
    cerr << "SimMainWindow::createSky : failed in reading decorations.iv" 
	 << endl;

  sceneinput.closeFile();

  return decor;
}

//---- End editable code block: End of generated code





















