
/////////////////////////////////////////////////////////////
//
// Source file for ViewBoardUI
//
//    This class implements the user interface created in 
//    RapidApp.
//
//    Restrict changes to those sections between
//    the "//--- Start/End editable code block" markers
//
//    This will allow RapidApp to integrate changes more easily
//
//    This class is a ViewKit user interface "component".
//    For more information on how components are used, see the
//    "ViewKit Programmers' Manual", and the RapidApp
//    User's Guide.
//
//
/////////////////////////////////////////////////////////////


#include "ViewBoardUI.h" // Generated header file for this class

#include <Xm/BulletinB.h> 
#include <Xm/Label.h> 
#include <Xm/PushB.h> 
#include <Xm/Scale.h> 
#include <Xm/ToggleB.h> 
#include <Vk/VkResource.h>
//---- Start editable code block: headers and declarations


//---- End editable code block: headers and declarations



// These are default resources for widgets in objects of this class
// All resources will be prepended by *<name> at instantiation,
// where <name> is the name of the specific instance, as well as the
// name of the baseWidget. These are only defaults, and may be overriden
// in a resource file by providing a more specific resource name

String  ViewBoardUI::_defaultViewBoardUIResources[] = {
        "*animToggle.labelString:  Animated View",
        "*frLabel.labelString:  frame rate:",
        "*frLabel1.labelString:  fps",
        "*label.labelString:  Animation Settings",
        "*pathToggle.labelString:  End effector path",
        "*playB.labelString:  <<",
        "*playE.labelString:  >>",
        "*playF.labelString:  >",
        "*playP.labelString:  ||",
        "*playR.labelString:  <",
        "*playSF.labelString:  >|",
        "*playSR.labelString:  |<",

        //---- Start editable code block: ViewBoardUI Default Resources


        //---- End editable code block: ViewBoardUI Default Resources

        (char*)NULL
};

ViewBoardUI::ViewBoardUI ( const char *name ) : VkComponent ( name ) 
{ 
    // No widgets are created by this constructor.
    // If an application creates a component using this constructor,
    // It must explictly call create at a later time.
    // This is mostly useful when adding pre-widget creation
    // code to a derived class constructor.

    //---- Start editable code block: ViewBoard constructor 2


    //---- End editable code block: ViewBoard constructor 2


}    // End Constructor




ViewBoardUI::ViewBoardUI ( const char *name, Widget parent ) : VkComponent ( name ) 
{ 
    //---- Start editable code block: ViewBoard pre-create


    //---- End editable code block: ViewBoard pre-create



    // Call creation function to build the widget tree.

     create ( parent );

    //---- Start editable code block: ViewBoard constructor


    //---- End editable code block: ViewBoard constructor


}    // End Constructor


ViewBoardUI::~ViewBoardUI() 
{
    // Base class destroys widgets

    //---- Start editable code block: ViewBoardUI destructor


    //---- End editable code block: ViewBoardUI destructor
}    // End destructor



void ViewBoardUI::create ( Widget parent )
{
  Arg      args[8];
  Cardinal count;
  count = 0;

  // Load any class-defaulted resources for this object
  
  setDefaultResources ( parent, _defaultViewBoardUIResources  );
    

    // Create an unmanaged widget as the top of the widget hierarchy

  _baseWidget = _viewBoard = XtVaCreateWidget ( _name,
						xmBulletinBoardWidgetClass,
						parent, 
						XmNresizePolicy, XmRESIZE_GROW, 
						(XtPointer) NULL ); 

    // install a callback to guard against unexpected widget destruction

    installDestroyHandler();


    // Create widgets used in this component
    // All variables are data members of this class

    _pathToggle = XtVaCreateManagedWidget  ( "pathToggle",
                                              xmToggleButtonWidgetClass,
                                              _baseWidget, 
                                              XmNalignment, XmALIGNMENT_BEGINNING, 
                                              XmNlabelType, XmSTRING, 
                                              XmNx, 175, 
                                              XmNy, 15, 
                                              XmNwidth, 136, 
                                              XmNheight, 26, 
                                              (XtPointer) NULL ); 

    XtAddCallback ( _pathToggle,
                    XmNvalueChangedCallback,
                    &ViewBoardUI::setPathToggleCallback,
                    (XtPointer) this ); 


    _playSR = XtVaCreateManagedWidget  ( "playSR",
                                          xmPushButtonWidgetClass,
                                          _baseWidget, 
                                          XmNlabelType, XmSTRING, 
                                          XmNx, 115, 
                                          XmNy, 145, 
                                          XmNwidth, 35, 
                                          XmNheight, 35, 
                                          (XtPointer) NULL ); 

    XtAddCallback ( _playSR,
                    XmNactivateCallback,
                    &ViewBoardUI::doPlaySRCallback,
                    (XtPointer) this ); 


    _playSF = XtVaCreateManagedWidget  ( "playSF",
                                          xmPushButtonWidgetClass,
                                          _baseWidget, 
                                          XmNlabelType, XmSTRING, 
                                          XmNx, 195, 
                                          XmNy, 145, 
                                          XmNwidth, 35, 
                                          XmNheight, 35, 
                                          (XtPointer) NULL ); 

    XtAddCallback ( _playSF,
                    XmNactivateCallback,
                    &ViewBoardUI::doPlaySFCallback,
                    (XtPointer) this ); 


    _playB = XtVaCreateManagedWidget  ( "playB",
                                         xmPushButtonWidgetClass,
                                         _baseWidget, 
                                         XmNlabelType, XmSTRING, 
                                         XmNx, 35, 
                                         XmNy, 145, 
                                         XmNwidth, 35, 
                                         XmNheight, 35, 
                                         (XtPointer) NULL ); 

    XtAddCallback ( _playB,
                    XmNactivateCallback,
                    &ViewBoardUI::doPlayBCallback,
                    (XtPointer) this ); 


    _playE = XtVaCreateManagedWidget  ( "playE",
                                         xmPushButtonWidgetClass,
                                         _baseWidget, 
                                         XmNlabelType, XmSTRING, 
                                         XmNx, 275, 
                                         XmNy, 145, 
                                         XmNwidth, 35, 
                                         XmNheight, 35, 
                                         (XtPointer) NULL ); 

    XtAddCallback ( _playE,
                    XmNactivateCallback,
                    &ViewBoardUI::doPlayECallback,
                    (XtPointer) this ); 


    _playP = XtVaCreateManagedWidget  ( "playP",
                                         xmPushButtonWidgetClass,
                                         _baseWidget, 
                                         XmNlabelType, XmSTRING, 
                                         XmNx, 155, 
                                         XmNy, 145, 
                                         XmNwidth, 35, 
                                         XmNheight, 35, 
                                         (XtPointer) NULL ); 

    XtAddCallback ( _playP,
                    XmNactivateCallback,
                    &ViewBoardUI::doPlayPCallback,
                    (XtPointer) this ); 


    _playR = XtVaCreateManagedWidget  ( "playR",
                                         xmPushButtonWidgetClass,
                                         _baseWidget, 
                                         XmNlabelType, XmSTRING, 
                                         XmNx, 75, 
                                         XmNy, 145, 
                                         XmNwidth, 35, 
                                         XmNheight, 35, 
                                         (XtPointer) NULL ); 

    XtAddCallback ( _playR,
                    XmNactivateCallback,
                    &ViewBoardUI::doPlayRCallback,
                    (XtPointer) this ); 


    _playF = XtVaCreateManagedWidget  ( "playF",
                                         xmPushButtonWidgetClass,
                                         _baseWidget, 
                                         XmNlabelType, XmSTRING, 
                                         XmNx, 235, 
                                         XmNy, 145, 
                                         XmNwidth, 35, 
                                         XmNheight, 35, 
                                         (XtPointer) NULL ); 

    XtAddCallback ( _playF,
                    XmNactivateCallback,
                    &ViewBoardUI::doPlayFCallback,
                    (XtPointer) this ); 


    _frScale = XtVaCreateManagedWidget  ( "frScale",
                                           xmScaleWidgetClass,
                                           _baseWidget, 
                                           XmNminimum, 1, 
                                           XmNorientation, XmHORIZONTAL, 
                                           XmNshowValue, True, 
                                           XmNx, 95, 
                                           XmNy, 90, 
                                           XmNwidth, 200, 
                                           XmNheight, 34, 
                                           (XtPointer) NULL ); 

    XtAddCallback ( _frScale,
                    XmNdragCallback,
                    &ViewBoardUI::setFrameRateCallback,
                    (XtPointer) this ); 


    _frLabel1 = XtVaCreateManagedWidget  ( "frLabel1",
                                            xmLabelWidgetClass,
                                            _baseWidget, 
                                            XmNlabelType, XmSTRING, 
                                            XmNx, 300, 
                                            XmNy, 105, 
                                            XmNwidth, 25, 
                                            XmNheight, 20, 
                                            (XtPointer) NULL ); 


    _frLabel = XtVaCreateManagedWidget  ( "frLabel",
                                           xmLabelWidgetClass,
                                           _baseWidget, 
                                           XmNlabelType, XmSTRING, 
                                           XmNx, 10, 
                                           XmNy, 105, 
                                           XmNwidth, 78, 
                                           XmNheight, 20, 
                                           (XtPointer) NULL ); 


    _label = XtVaCreateManagedWidget  ( "label",
                                         xmLabelWidgetClass,
                                         _baseWidget, 
                                         XmNlabelType, XmSTRING, 
                                         XmNx, 10, 
                                         XmNy, 60, 
                                         XmNwidth, 136, 
                                         XmNheight, 20, 
                                         (XtPointer) NULL ); 


    _animToggle = XtVaCreateManagedWidget  ( "animToggle",
                                              xmToggleButtonWidgetClass,
                                              _baseWidget, 
                                              XmNalignment, XmALIGNMENT_BEGINNING, 
                                              XmNlabelType, XmSTRING, 
                                              XmNindicatorOn, True, 
                                              XmNx, 10, 
                                              XmNy, 20, 
                                              XmNwidth, 120, 
                                              XmNheight, 20, 
                                              (XtPointer) NULL ); 

    XtAddCallback ( _animToggle,
                    XmNvalueChangedCallback,
                    &ViewBoardUI::setAnimToggleCallback,
                    (XtPointer) this ); 



    //---- Start editable code block: ViewBoardUI create


    //---- End editable code block: ViewBoardUI create
}

const char * ViewBoardUI::className()
{
    return ("ViewBoardUI");
}    // End className()




/////////////////////////////////////////////////////////////// 
// The following functions are static member functions used to 
// interface with Motif.
/////////////////////////////////// 

void ViewBoardUI::doPlayBCallback ( Widget    w,
                                    XtPointer clientData,
                                    XtPointer callData ) 
{ 
    ViewBoardUI* obj = ( ViewBoardUI * ) clientData;
    obj->doPlayB ( w, callData );
}

void ViewBoardUI::doPlayECallback ( Widget    w,
                                    XtPointer clientData,
                                    XtPointer callData ) 
{ 
    ViewBoardUI* obj = ( ViewBoardUI * ) clientData;
    obj->doPlayE ( w, callData );
}

void ViewBoardUI::doPlayFCallback ( Widget    w,
                                    XtPointer clientData,
                                    XtPointer callData ) 
{ 
    ViewBoardUI* obj = ( ViewBoardUI * ) clientData;
    obj->doPlayF ( w, callData );
}

void ViewBoardUI::doPlayPCallback ( Widget    w,
                                    XtPointer clientData,
                                    XtPointer callData ) 
{ 
    ViewBoardUI* obj = ( ViewBoardUI * ) clientData;
    obj->doPlayP ( w, callData );
}

void ViewBoardUI::doPlayRCallback ( Widget    w,
                                    XtPointer clientData,
                                    XtPointer callData ) 
{ 
    ViewBoardUI* obj = ( ViewBoardUI * ) clientData;
    obj->doPlayR ( w, callData );
}

void ViewBoardUI::doPlaySFCallback ( Widget    w,
                                     XtPointer clientData,
                                     XtPointer callData ) 
{ 
    ViewBoardUI* obj = ( ViewBoardUI * ) clientData;
    obj->doPlaySF ( w, callData );
}

void ViewBoardUI::doPlaySRCallback ( Widget    w,
                                     XtPointer clientData,
                                     XtPointer callData ) 
{ 
    ViewBoardUI* obj = ( ViewBoardUI * ) clientData;
    obj->doPlaySR ( w, callData );
}

void ViewBoardUI::setAnimToggleCallback ( Widget    w,
                                          XtPointer clientData,
                                          XtPointer callData ) 
{ 
    ViewBoardUI* obj = ( ViewBoardUI * ) clientData;
    obj->setAnimToggle ( w, callData );
}

void ViewBoardUI::setFrameRateCallback ( Widget    w,
                                         XtPointer clientData,
                                         XtPointer callData ) 
{ 
    ViewBoardUI* obj = ( ViewBoardUI * ) clientData;
    obj->setFrameRate ( w, callData );
}

void ViewBoardUI::setPathToggleCallback ( Widget    w,
                                          XtPointer clientData,
                                          XtPointer callData ) 
{ 
    ViewBoardUI* obj = ( ViewBoardUI * ) clientData;
    obj->setPathToggle ( w, callData );
}



/////////////////////////////////////////////////////////////// 
// The following functions are called from the menu items 
// in this window.
/////////////////////////////////// 

void ViewBoardUI::doPlayB ( Widget, XtPointer ) 
{
    // This virtual function is called from doPlayBCallback.
    // This function is normally overriden by a derived class.

}

void ViewBoardUI::doPlayE ( Widget, XtPointer ) 
{
    // This virtual function is called from doPlayECallback.
    // This function is normally overriden by a derived class.

}

void ViewBoardUI::doPlayF ( Widget, XtPointer ) 
{
    // This virtual function is called from doPlayFCallback.
    // This function is normally overriden by a derived class.

}

void ViewBoardUI::doPlayP ( Widget, XtPointer ) 
{
    // This virtual function is called from doPlayPCallback.
    // This function is normally overriden by a derived class.

}

void ViewBoardUI::doPlayR ( Widget, XtPointer ) 
{
    // This virtual function is called from doPlayRCallback.
    // This function is normally overriden by a derived class.

}

void ViewBoardUI::doPlaySF ( Widget, XtPointer ) 
{
    // This virtual function is called from doPlaySFCallback.
    // This function is normally overriden by a derived class.

}

void ViewBoardUI::doPlaySR ( Widget, XtPointer ) 
{
    // This virtual function is called from doPlaySRCallback.
    // This function is normally overriden by a derived class.

}

void ViewBoardUI::setAnimToggle ( Widget, XtPointer ) 
{
    // This virtual function is called from setAnimToggleCallback.
    // This function is normally overriden by a derived class.

}

void ViewBoardUI::setFrameRate ( Widget, XtPointer ) 
{
    // This virtual function is called from setFrameRateCallback.
    // This function is normally overriden by a derived class.

}

void ViewBoardUI::setPathToggle ( Widget, XtPointer ) 
{
    // This virtual function is called from setPathToggleCallback.
    // This function is normally overriden by a derived class.

}



//---- Start editable code block: End of generated code


//---- End editable code block: End of generated code
