//////////////////////////////////////////////////////////////
//
// Source file for ViewMainWindow
//
//    This class is a subclass of VkSimpleWindow
//               One robot
/////////////////////////////////////////////////////////////

#include "ViewMainWindow.h"

#include <Vk/VkApp.h>
#include <Vk/VkResource.h>


// Externally defined classes referenced by this class: 

#include "ViewBoard.h"

extern void VkUnimplemented ( Widget, const char * );

//---- Start editable code block: headers and declarations

#include <iostream.h>
#include <Inventor/sensors/SoTimerSensor.h>
#include <Inventor/nodes/SoSwitch.h>
#include <Inventor/nodes/SoSeparator.h>
#include "ivsimulator.h"

static void animCB(void *data, SoSensor *);

//---- End editable code block: headers and declarations



// These are default resources for widgets in objects of this class
// All resources will be prepended by *<name> at instantiation,
// where <name> is the name of the specific instance, as well as the
// name of the baseWidget. These are only defaults, and may be overriden
// in a resource file by providing a more specific resource name

String  ViewMainWindow::_defaultViewMainWindowResources[] = {

        //---- Start editable code block: ViewMainWindow Default Resources


        //---- End editable code block: ViewMainWindow Default Resources

        (char*)NULL
};


//---- Class declaration

ViewMainWindow::ViewMainWindow ( const char *name,
                                ArgList args,
                                Cardinal argCount) : 
                                VkSimpleWindow ( name, args, argCount )
{
    // Load any class-default resources for this object

    setDefaultResources ( baseWidget(), _defaultViewMainWindowResources  );


    // Create the view component contained by this window

    _viewBoard = new ViewBoard ( "viewBoard",mainWindowWidget() );


    XtVaSetValues ( _viewBoard->baseWidget(),
                    XmNwidth, 339, 
                    XmNheight, 324, 
                    (XtPointer) NULL );

    // Add the component as the main view

    addView ( _viewBoard );

    // Set up the window manager properties for this window.
    // This window is a coprimary window, as defined by the SGI Style Guide
    // This window has no title bar.

     VkConfigureWM ( this,
                    FALSE,   //quit
                    TRUE,  // close
                    TRUE,   // border
                    FALSE,  // title
                    TRUE,   // allow resize
                    FALSE,  // allow iconify
                    FALSE);  // Show window manager menu

		 //---- Start editable code block: ViewMainWindow constructor

		 _viewBoard->setViewer(this);
		 anim = new animS;
		 anim->sim = NULL;
		 anim->wp = NULL;
		 anim->owp = NULL;
		 anim->valid = false;
		 anim->enabled = false;
		 animSensor = new SoTimerSensor(animCB, anim);
		 animSensor->unschedule();

    //---- End editable code block: ViewMainWindow constructor


}    // End Constructor


ViewMainWindow::~ViewMainWindow()
{
    delete _viewBoard;
    //---- Start editable code block: ViewMainWindow destructor


    //---- End editable code block: ViewMainWindow destructor
}    // End destructor


const char *ViewMainWindow::className()
{
    return ("ViewMainWindow");
}    // End className()


Boolean ViewMainWindow::okToQuit()
{
    //---- Start editable code block: ViewMainWindow okToQuit



    // This member function is called when the user quits by calling
    // theApplication->terminate() or uses the window manager close protocol
    // This function can abort the operation by returning FALSE, or do some.
    // cleanup before returning TRUE. The actual decision is normally passed on
    // to the view object

    return ( TRUE );
    //---- End editable code block: ViewMainWindow okToQuit
}    // End okToQuit()





/////////////////////////////////////////////////////////////// 
// The following functions are called from callbacks 
/////////////////////////////////// 

//---- Start editable code block: End of generated code

static bool stepAnim(animS *anim) {
	Milestone cur;
	
	if (anim->dir) {
	  // Animate the path starting from the beginning
	  
	  if (anim->res) {
	    anim->res = false;
	    anim->it = anim->p.begin();
	  }
	  // Or starting from the end
	  if (anim->it == anim->p.end()){
	    anim->res = true;
	    anim->dir = false;
	    return false;
	  }
	    cur = *anim->it++;
	} else {
	  if (anim->res) {
	    anim->res = false;
	    anim->it = anim->p.end();
	  
	    anim->it--;
	  }
	  if (anim->it == anim->p.begin()) {
	    anim->res = true;
	    anim->dir = true;
	  }
	  cur = *anim->it--;
	}
	
	anim->sim->SetConfig(cur.getConfig(),0);
	return !anim->res;
}


static void animCB(void *data, SoSensor *sensor) {
	animS *anim = (animS *)data;

	if (!stepAnim(anim)) {
		sensor->unschedule();
	}
}

bool ViewMainWindow::togglePath(bool newVal) {
	if (newVal) {
		if (anim->wp) {
			anim->sim->getRoot()->addChild(anim->wp);
			if (anim->owp) {
				anim->sim->getRoot()->addChild(anim->owp);
			}
			return true;
		} else {
			return false;
		}
	}
	
	if (anim->wp)
		anim->sim->getRoot()->removeChild(anim->wp);
	if (anim->owp)
		anim->sim->getRoot()->removeChild(anim->owp);

	return false;
}

bool ViewMainWindow::toggleAnimation(bool newVal) {
	if (newVal) {
		if (anim->valid) {
			anim->sim->GetConfig(&q,0);
			anim->enabled = true;
			stepAnim(anim);
			return true;
		} else {
			return false;
		}
	}

	anim->enabled = false;
	anim->sim->SetConfig(q,0);
	
	return false;
}

void ViewMainWindow::animCmd(int cmd) {
	if (anim->valid && anim->enabled) {
		switch (cmd) {
		case 0: {
			animSensor->unschedule();
			anim->res = true;
			anim->dir = true;
			stepAnim(anim);
			break;
		}
		
		case 1: {
			anim->dir = false;
			animSensor->schedule();
			break;
		}
		
		case 2: {
			animSensor->unschedule();
			anim->dir = false;
			stepAnim(anim);
			break;
		}
		
		case 3: {
			animSensor->unschedule();
			break;
		}
		
		case 4: {
			animSensor->unschedule();
			anim->dir = true;
			stepAnim(anim);
			break;
		}
		
		case 5: {
			anim->dir = true;
			animSensor->schedule();
			break;
		}
		case 6: {
			animSensor->unschedule();
			anim->res = true;
			anim->dir = false;
			stepAnim(anim);
			break;
		}
		
		default: break;
		}
	}
}

void ViewMainWindow::setFrameRate(int newRate) {
	animSensor->setInterval(1./1000.);
				//newRate);
}

void ViewMainWindow::setSim(IVSimulator *sim) {
	anim->sim = sim;
}

void ViewMainWindow::setPath(const Path &p, SoSwitch *wp_) {
	if (anim->owp) {
		anim->sim->getRoot()->removeChild(anim->owp);
		anim->owp->unref();
	}
	anim->owp = anim->wp;
	if (anim->owp)
		anim->owp->whichChild = 0;
	if (wp_) {
		anim->wp = wp_;
		anim->wp->ref();
		anim->wp->ref();
		anim->wp->whichChild = SO_SWITCH_ALL;
	}
	anim->p = p;
	anim->it = anim->p.begin();
	anim->res = true;
	anim->dir = true;
	anim->valid = true;
}

//---- End editable code block: End of generated code








