#include "baseivsimulator.h"

#include "joint.h"
#include "jointkit.h"
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTransform.h>

#include <iostream.h>

BaseIVSimulator::BaseIVSimulator(SoSeparator *root_):
  IVSimulator(root_) {

  for(int k=0;k<ROBOTS;k++)
    {
      baseXform[k] = new SoTransform;
      baseXform[k]->ref();
      robotSep[k]->insertChild(baseXform[k], 0);
      basePose[k].makeIdentity();
    }  
}

BaseIVSimulator::~BaseIVSimulator() {}

const Pose&
BaseIVSimulator::GetBase()
{
  return basePose[0];
}

void BaseIVSimulator::SetBase(const Pose& basePose_) {
  for(int i=0;i<10;i++)
    basePose[i] = basePose_;
  
  SbRotation rotR[10];
  SbVec3f transR[10];
  
  /*
    The positions and orientations of the robots 
    (up to six robots)

    However, only the number of robots specified on
    ivsimulator are going to be shown.

          Robot| Translation | Rotation
          --------------------------------
           R0  |  0,0,  0    |  0,0,0,0
           R1  |  0,0,-1.5   |  0,0,0,0
           R2  |  0,0,-2.5   |  0,1,0,0
           R3  |  2,0,-2.5   |  0,1,0,M_PI
           R4  |  2,0,-1.5   |  0,1,0,M_PI
           R5  |  2,0, 0     |  0,1,0,M_PI
  */

  rotR[0].setValue(SbVec3f(0.0, 0.0, 0.0), 0);
  rotR[1].setValue(SbVec3f(0.0, 0.0, 0.0), 0);
  rotR[2].setValue(SbVec3f(0.0, 0.0, 0.0), 0);
  rotR[3].setValue(SbVec3f(0.0, 1.0, 0.0), M_PI);
  rotR[4].setValue(SbVec3f(0.0, 1.0, 0.0), M_PI);
  rotR[5].setValue(SbVec3f(0.0, 1.0, 0.0), M_PI);
  
  transR[0].setValue(-0.2, 0.0,  0.0);
  transR[1].setValue(-0.2, 0.0, -1.5 );
  transR[2].setValue(-0.2, 0.0, -2.0);
  transR[3].setValue(2.2,  0.0, -2.0);
  transR[4].setValue(2.2,  0.0, -1.5);
  transR[5].setValue(2.2,  0.0,  0.0);

  int k;
  for(k=0;k<ROBOTS;k++){
    // Applying the transformations to the robots
    baseXform[k]->rotation = rotR[k];
    baseXform[k]->translation=transR[k];
    basePose[k].setRotate(rotR[k]); 
    basePose[k].setTranslate(transR[k]);
    
    if (jointKitsR[k].getLength() > 0) {
      JointKit *jk =
	(JointKit *)(((SoNodeKitPath *)jointKitsR[k][0])->getNodeFromTail(0));
      jk->accuMatrix(basePose[k]);
      jk->updateMatrix(true);
    }
  }
  
  // For the back robots
  for (k=0;k<2;k++){
    if (brJointKits[k].getLength() > 0) {
      JointKit *jk = 
	(JointKit *)(((SoNodeKitPath *)brJointKits[k][0])->getNodeFromTail(0));
      jk->accuMatrix(basePose[k]);
      jk->updateMatrix(true);
    }
  }
}












































