#include "joint.h"
#include "GenCoords.h"
#include "jointengine.h"
#include "jointinvengine.h"
#include <assert.h>

SO_NODE_ABSTRACT_SOURCE(Joint);

void Joint::initClass() {
  SO_NODE_INIT_ABSTRACT_CLASS(Joint, SoTransform, "Transform");
}

Joint::Joint() {
  SO_NODE_CONSTRUCTOR(Joint);

  SO_NODE_ADD_FIELD(val, (0));
  SO_NODE_ADD_FIELD(min, (0));
  SO_NODE_ADD_FIELD(max, (0));
  SO_NODE_ADD_FIELD(def, (0));
  SO_NODE_ADD_FIELD(vmax, (0));

  engine = NULL;
  invEngine = NULL;
}

Joint::~Joint() {
	if (engine)
		engine->unref();
	if (invEngine)
		invEngine->unref();
}

bool Joint::getVals(GenCoords *dest, int &idx) {
  	for (int i = 0; i < getNumVals(); i++) {
		if (idx < dest->Dim()) {
			(*dest)[idx] = getValAt(i);
		}
		idx++;
	}

	return (idx <= dest->Dim());
}

int Joint::getActVals(double *dest, int &idx) {
  int i=0;
	for (i = 0; i < getNumVals(); i++) {
		dest[idx] = getActValAt(i);
		idx++;
	}
	return i;
}

bool Joint::setVals(const GenCoords *src, int &idx) {
  int i;
	for (i = 0; i < getNumVals(); i++) {
		if (idx < src->Dim()) {
			setValAt(i, (*src)[idx]);
		}
		idx++;
	}
	
	return (idx <= src->Dim());
}

int Joint::setActVals(const double *src, int &idx) {
  int i;
	for (i = 0; i < getNumVals(); i++) {
		setActValAt(i, src[idx]);
		idx++;
	}

	return i;
}

double Joint::getValAt(int i) {
	if ((i >= 0) && (i < getNumVals())) {
		return ((val[i]-min[i])/(max[i]-min[i]));
	} else 
		return -1;
}
	
double Joint::getActValAt(int i) {
	if ((i >= 0) && (i < getNumVals()))
		return val[i];
	else 
		return -1;
}

bool Joint::setValAt(int i, double newVal) {
	if ((i >= 0) && (i < getNumVals())) {
		val.set1Value(i, (newVal*(max[i]-min[i])+min[i]));
		return true;
	} else 
		return false;
}

bool Joint::setActValAt(int i, double newVal) {
	if ((i >= 0) && (i < getNumVals())) {
		val.set1Value(i, newVal);
		return true;
	} else 
		return false;
}

bool Joint::getMaxs(GenCoords *dest, int &idx)
{
  int i;
  for (i=0; i < getNumVals(); i++) {
    if ( idx < dest->Dim() ) {
      (*dest)[idx] = getMaxAt(i);
    }
    idx++;
  }

  return (idx <= dest->Dim());
}

bool Joint::getMins(GenCoords *dest, int &idx)
{
  int i;
  for (i=0; i < getNumVals(); i++) {
    if ( idx < dest->Dim() ) {
      (*dest)[idx] = getMinAt(i);
    }
    idx++;
  }

  return (idx <= dest->Dim());
}

double Joint::getMaxAt(int i) {
  if ( i >=0 && i < getNumVals() ) 
    return max[i];
  else
    return -1;
}

double Joint::getMinAt(int i) {
  if ( i >=0 && i < getNumVals() ) 
    return min[i];
  else
    return -1;
}


bool Joint::getVmaxs(GenCoords *dest, int &idx)
{
  int i;
  for (i=0; i < getNumVals(); i++) {
    if ( idx < dest->Dim() ) {
      (*dest)[idx] = getVmaxAt(i);
    }
    idx++;
  }

  return (idx <= dest->Dim());
}

int Joint::getActVmaxs(double *dest, int &idx)
{
  int i;
  for (i=0; i < getNumVals(); i++) {
    dest[idx] = getActValAt(i);
    idx++;
  }
  
  return i;
}


double Joint::getVmaxAt(int i) {
  if ( i >=0 && i < getNumVals() ) 
    return vmax[i] / (max[i]-min[i]);
  else
    return -1;
}

double Joint::getActVmaxAt(int i) {
  if ( i >=0 && i < getNumVals() ) 
    return vmax[i];
  else
    return -1;
}
