#include "jointkit.h"

#include <iostream.h>
#include "joint.h"
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTransform.h>

SO_KIT_SOURCE(JointKit);

void JointKit::initClass() {
  SO_KIT_INIT_CLASS(JointKit, SoBaseKit, "BaseKit");
}

JointKit::JointKit() {
  SO_KIT_CONSTRUCTOR(JointKit);

  SO_KIT_ADD_CATALOG_ENTRY(topSeparator, SoSeparator, TRUE, this,, FALSE);
  SO_KIT_ADD_CATALOG_ENTRY(parentTrans, SoTransform, TRUE, topSeparator,,
					  TRUE);
  SO_KIT_ADD_CATALOG_ABSTRACT_ENTRY(joint, Joint, PrisJoint, FALSE, 
							 topSeparator,, TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(localTrans, SoTransform, TRUE, topSeparator,, TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(link, Link, TRUE, topSeparator,, TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(children, SoSeparator, TRUE, topSeparator,, TRUE);

  SO_KIT_INIT_INSTANCE();

  setSearchingChildren(TRUE);
	upMatrix.makeIdentity();
}

JointKit::~JointKit() {}

void JointKit::updateMatrix(bool propagate) {
  SoTransform *pt, *jt, *lt;
  SbVec3f uv(1, 1, 1);
  
  locMatrix.makeIdentity();
  
  pt = (SoTransform *)getPart("parentTrans", false);
  jt = (SoTransform *)getPart("joint", false);
  lt = (SoTransform *)getPart("localTrans", false);
  
  if (pt) {
    SbMatrix ptm;
    
    ptm.setTransform(pt->translation.getValue(), pt->rotation.getValue(), uv);
    locMatrix.multLeft(ptm);
  }
  
  if (jt) {
    SbMatrix jtm;
    
    jtm.setTransform(jt->translation.getValue(), jt->rotation.getValue(), uv);
    locMatrix.multLeft(jtm);
  }
  
  if (lt) {
    SbMatrix ltm;
    
    ltm.setTransform(lt->translation.getValue(), lt->rotation.getValue(), uv);
    locMatrix.multLeft(ltm);
  }
  
  accMatrix = upMatrix;
  accMatrix.multLeft(locMatrix);
  if (propagate)
    updateChildren();
}

void JointKit::updateChildren() {
	SoSeparator *children = (SoSeparator *)getPart("children", false);

	if (children) {
		for (int i = 0; i < children->getNumChildren(); i++) {
			SoNode *child = children->getChild(i);
			if (child->isOfType(JointKit::getClassTypeId())) {
				((JointKit *)child)->accuMatrix(accMatrix);
			}
		}
	}
}

void JointKit::accuMatrix(const SbMatrix & upMatrix_) {
	upMatrix = upMatrix_;

	accMatrix = upMatrix;
	accMatrix.multLeft(locMatrix);

	updateChildren();
}






