#include <iostream.h>

#include <Inventor/actions/SoCallbackAction.h>
#include <Inventor/actions/SoGetMatrixAction.h>
#include <Inventor/nodes/SoShape.h>
#include <Inventor/SoNodeKitPath.h>
#include <Inventor/SoPrimitiveVertex.h>


#include "link.h"
#include "jointkit.h"

SO_NODE_SOURCE(Link);

void Link::initClass() {
  SO_NODE_INIT_CLASS(Link, SoSeparator, "Separator");
}

Link::Link() {
  SO_NODE_CONSTRUCTOR(Link);
  
  colModel = NULL;
  oldObjModel = NULL;
  dist = 10e10;
  tol = 1;
}

Link::~Link() {
  delete colModel;
}

void Link::buildColModel(JointKit *parent_) {
  parent = parent_;
  SoCallbackAction triAction;
  
  colModel = new PQP_Model;
  colModel->BeginModel();
  triAction.addTriangleCallback(SoShape::getClassTypeId(),
				_TriangulateCB, colModel);
  triAction.apply(this);
  colModel->EndModel();
  //  colModel->MemUsage(1);
}

PQP_Model *Link::getColModel()
{
  setColTransform(); 
  return colModel;
}

void Link::setColTransform() {

  SbMatrix matrix = parent->getAccuMatrix();
  
  // In this function, we read the info for position/orientation
  // and write it directly to the rotation matrix and translation vector
  for (int i = 0; i < 3; i++) { 
    for (int j = 0; j < 3; j++) {
      rot[i][j] = matrix[j][i];
    }
      trans[i] = matrix[3][i];
  }
}


double Link::getCurDist(Obstacle *objModel, double tol_) {
  oldObjModel = objModel->getColModel();
  
  if (!objModel)
    dist = 10e10;
  else {
    PQP_DistanceResult dres;
    double rel_err = 0.0, abs_err = 0.0;
    setColTransform(); 
    
    PQP_Distance(&dres,rot,trans,colModel,
		 objModel->rot, objModel->trans, objModel->colModel,
		 rel_err,abs_err);
    dist = dres.Distance();
  } 
  return dist;
}

bool Link::isClear(Obstacle *objModel) {
  oldObjModel = objModel->getColModel();

  PQP_CollideResult cres;
  setColTransform();
  PQP_Collide(&cres,rot,trans,colModel,
	      objModel->rot, objModel->trans,objModel->colModel, 2);
  int res = cres.Colliding();
  if (res==1) 
    return false;
  else
    return true;
}

bool Link::isClear(Link* link2) {
  PQP_CollideResult cres;

  setColTransform(); 
  PQP_Collide(&cres,rot,trans,colModel,
	      link2->rot, link2->trans,link2->colModel,2);
  int res = cres.Colliding();
  if (res==1) 
    return false;
  else
    return true;
}

void Link::_TriangulateCB(void *data, SoCallbackAction *,
			  const SoPrimitiveVertex *vertex1,
			  const SoPrimitiveVertex *vertex2,
			  const SoPrimitiveVertex *vertex3) 
{
  static long cnt = 0;
  //    cerr <<cnt++<<endl;
  PQP_Model *colModel = ( PQP_Model *)data;
  
  SbVec3f points[3];
  
  points[0] = vertex1->getPoint(); 
  points[1] = vertex2->getPoint(); 
  points[2] = vertex3->getPoint(); 
  
  if (colModel) {
    PQP_REAL verts[3][3];
    
    for (int i = 0; i < 3; i++)
      for (int j = 0; j < 3; j++)
	verts[i][j] = points[i][j];

    colModel->AddTri(verts[0], verts[1], verts[2],cnt++);
  }
}










